import * as React from 'react';
import { ensureEnum, brushToString, stringToBrush, ensureBool, initializePropertiesFromCss, NamePatcher, isValidProp, toSpinal, getModifiedProps } from "igniteui-react-core";
import { FontInfo } from "igniteui-react-core";
import { ReactRenderer, ReactWrapper } from "igniteui-react-core";
import { Base, delegateCombine, delegateRemove } from "igniteui-react-core";
import { ListPanel } from './ListPanel';
import { SyncableObservableCollection$1 } from "igniteui-react-core";
import { IgrListPanelTemplateItemUpdatingEventArgs } from './igr-list-panel-template-item-updating-event-args';
import { ListPanelOrientation_$type } from './ListPanelOrientation';
import { IgrListPanelContentRefreshedEventArgs } from './igr-list-panel-content-refreshed-event-args';
import { IgrListPanelTemplateWidthRequestedEventArgs } from './igr-list-panel-template-width-requested-event-args';
import { IgrListPanelTemplateHeightRequestedEventArgs } from './igr-list-panel-template-height-requested-event-args';
import { LocalDataSource } from "igniteui-react-core";
import { Scroller } from "igniteui-react-core";
import { ScrollbarStyle_$type } from "igniteui-react-core";
import { ListPanelScrollerKeyboardListener } from './ListPanelScrollerKeyboardListener';
import { IgrListPanelItemEventArgs } from './igr-list-panel-item-event-args';
import { IgrListPanelActiveRowChangedEventArgs } from './igr-list-panel-active-row-changed-event-args';
import { IgrListPanelSelectionChangedEventArgs } from './igr-list-panel-selection-changed-event-args';
import { IgrListPanelSelectedKeysChangedEventArgs } from './igr-list-panel-selected-keys-changed-event-args';
import { IgrListPanelSelectedItemsChangedEventArgs } from './igr-list-panel-selected-items-changed-event-args';
import { IgrListPanelPrimaryKeyValue } from './igr-list-panel-primary-key-value';
import { IgrListPanelSelectedKeysCollection } from './igr-list-panel-selected-keys-collection';
import { ListPanelPrimaryKeyValue } from './ListPanelPrimaryKeyValue';
import { ListPanelSelectedKeysCollection as ListPanelSelectedKeysCollection_internal } from './ListPanelSelectedKeysCollection';
import { ListPanelSelectionMode_$type } from './ListPanelSelectionMode';
import { ListPanelActivationMode_$type } from './ListPanelActivationMode';
import { ListPanelSelectionBehavior_$type } from './ListPanelSelectionBehavior';
import { IgrListPanelSelectedItemsCollection } from './igr-list-panel-selected-items-collection';
import { ListPanelSelectedItemsCollection as ListPanelSelectedItemsCollection_internal } from './ListPanelSelectedItemsCollection';
export class IgrListPanel extends React.Component {
    setScrollbarStyle(style) {
        if (this._scroller) {
            this._scroller.scrollbarStyle = style;
        }
    }
    setScrollbarColor(color) {
        if (this._scroller) {
            this._scroller.scrollbarBrush = color;
        }
    }
    createLocalDataSource(itemsSource, schemaIncludedProperties) {
        let lds = new LocalDataSource();
        lds.schemaIncludedProperties = schemaIncludedProperties;
        lds.dataSource = itemsSource;
        return lds;
    }
    onContentSizeChanged() {
        var width = this.i.absoluteWidth;
        var height = this.i.absoluteHeight;
        if (this._scroller != null &&
            !isNaN(width) && !isNaN(height)) {
            this._scroller.contentWidth = width;
            this._scroller.contentHeight = height;
        }
    }
    set height(value) {
        this._height = value;
        if (this._elRef) {
            this._elRef.style.height = value;
            this.notifySizeChanged();
        }
    }
    get height() {
        return this._height;
    }
    set width(value) {
        this._width = value;
        if (this._elRef) {
            this._elRef.style.height = value;
            this.notifySizeChanged();
        }
    }
    get width() {
        return this._width;
    }
    ensureStretch() {
        this.width = "100%";
        this.height = "100%";
    }
    _getMainRef(ref) {
        this._elRef = ref;
        this.verifyReferences();
    }
    render() {
        let div = React.createElement("div", {
            ref: this._getMainRef,
            className: "ig-list-panel igr-list-panel",
        });
        return div;
    }
    moveViewportTo(x, y) {
        this._scroller.scrollTo(x, y);
    }
    verifyReferences() {
        if (this._elRef) {
            this._reactRenderer = new ReactRenderer(this._elRef, document, true, {});
            if (document) {
                this._elRef.style.display = "block";
                this._elRef.style.width = "100%";
                this._elRef.style.height = "100%";
            }
            // render needs to fire again after the portalManager is created. 
            this.i.provideContainer(this._reactRenderer);
            let scroller = new Scroller();
            scroller.scrollbarStyle = this.scrollbarStyle;
            this._scroller = scroller;
            this._scroller.keyboardListener = new ListPanelScrollerKeyboardListener(this.i);
            scroller.provideContainer(this._reactRenderer);
            this._content = (this.i.view.contentGrid);
            scroller.provideContent(new ReactWrapper(this._content, this._reactRenderer));
            scroller.onScrolling = () => this.onScroll();
            scroller.onScrollStart = () => this.onScrollStart();
            scroller.onScrollStop = () => this.onScrollStop();
            scroller.viewportWidth = this._reactRenderer.rootWrapper.width();
            scroller.viewportHeight = this._reactRenderer.rootWrapper.height();
            this.notifySizeChanged();
            this._reactRenderer.addSizeWatcher(() => {
                this.notifySizeChanged();
            });
        }
    }
    onScroll() {
        let top = this._scroller.scrollTop;
        let left = this._scroller.scrollLeft;
        let width = this._scroller.viewportWidth;
        let height = this._scroller.viewportHeight;
        //console.log("top:" + top);
        this.i.setViewport(left, top, left + width, top + height);
    }
    onScrollStart() {
        //this.i.notifyScrollStart();
    }
    onScrollStop() {
        //this.i.notifyScrollStop();
    }
    scrollTo(x, y) {
        this._scroller.scrollTo(x, y);
        //let width = this._scroller.viewportWidth;
        //let height = this._scroller.viewportHeight;
        //this._grid.setViewport(x, y, x + width, y + height);
    }
    constructor(props) {
        super(props);
        this._implementation = null;
        this._reactRenderer = null;
        this._scroller = null;
        this._mounted = true;
        this._selectedItems = null;
        this._selectedKeys = null;
        this.__p = null;
        this._hasUserValues = new Set();
        this._stylingContainer = null;
        this._stylingParent = null;
        this._inStyling = false;
        this._selectedItemsChanged = null;
        this._selectedItemsChanged_wrapped = null;
        this._selectedKeysChanged = null;
        this._selectedKeysChanged_wrapped = null;
        this._selectionChanged = null;
        this._selectionChanged_wrapped = null;
        this._activeRowChanged = null;
        this._activeRowChanged_wrapped = null;
        this._contentRefreshed = null;
        this._contentRefreshed_wrapped = null;
        this._rowUpdating = null;
        this._rowUpdating_wrapped = null;
        this._itemWidthRequested = null;
        this._itemWidthRequested_wrapped = null;
        this._itemHeightRequested = null;
        this._itemHeightRequested_wrapped = null;
        this._itemClicked = null;
        this._itemClicked_wrapped = null;
        this._actualPrimaryKeyChange = null;
        this._actualPrimaryKeyChange_wrapped = null;
        if (this._styling) {
            NamePatcher.ensureStylablePatched(Object.getPrototypeOf(this));
        }
        this._implementation = this.createImplementation();
        this._implementation.viewportManager = this;
        this._implementation.externalListPanel = this;
        this._implementation.externalObject = this;
        this._getMainRef = this._getMainRef.bind(this);
        //var multiColumnComboBox = this.i;
        //multiColumnComboBox.notifySizeChanged();
        if (props) {
            this.initializeProperties();
        }
        this._initialized = true;
    }
    shouldComponentUpdate(nextProps, nextState) {
        const mod = getModifiedProps(this.props, nextProps);
        for (const p of Object.keys(mod)) {
            if (isValidProp(this, p)) {
                this[p] = mod[p];
            }
        }
        return true;
    }
    initializeProperties() {
        for (const p of Object.keys(this.props)) {
            if (isValidProp(this, p)) {
                this[p] = this.props[p];
            }
        }
    }
    // supports angular themes or custom properties set in CSS
    updateStyle() {
        this._styling(this._elRef, this);
    }
    componentWillUnmount() {
        if (this._mounted) {
            this._mounted = false;
            if (this.i) {
                if (this._scroller) {
                    this._scroller.onDetachedFromUI();
                }
            }
        }
        this.i.destroy();
        if (this._reactRenderer)
            this._reactRenderer.destroy();
    }
    componentDidMount() {
        if (!this._mounted) {
            this._mounted = true;
            if (this.i) {
                if (this._scroller) {
                    this._scroller.onAttachedToUI();
                }
            }
        }
        this._elRef.style.width = this._width ? this._width : "";
        this._elRef.style.height = this._height ? this._height : "";
        this.notifySizeChanged();
        this.initializeContent();
    }
    notifySizeChanged() {
        if (this._reactRenderer) {
            var width = this._reactRenderer.rootWrapper.width();
            var height = this._reactRenderer.rootWrapper.height();
            this._scroller.viewportWidth = width;
            this._scroller.viewportHeight = height;
            this._scroller.onViewportSizeChanged(width, height, true);
        }
        this.i.notifySizeChanged();
    }
    initializeContent() {
        this._styling(this._elRef, this);
        this.updateStyle();
        this.notifySizeChanged();
    }
    createImplementation() {
        return new ListPanel();
    }
    get i() {
        return this._implementation;
    }
    /**
 * Sets or gets the grid selection mode.
*/
    get selectionMode() {
        return this.i.aq;
    }
    set selectionMode(v) {
        this.i.aq = ensureEnum(ListPanelSelectionMode_$type, v);
    }
    /**
     * Gets or sets whether the active row style will be applied to rows when made active.
    */
    get isActiveRowStyleEnabled() {
        return this.i.bv;
    }
    set isActiveRowStyleEnabled(v) {
        this.i.bv = ensureBool(v);
    }
    /**
     * Gets or sets the background color to use for the cells when they are selected
    */
    get selectedBackground() {
        return brushToString(this.i.hz);
    }
    set selectedBackground(v) {
        this.i.hz = stringToBrush(v);
    }
    /**
     * Gets or sets the background color to use.
    */
    get normalBackground() {
        return brushToString(this.i.hx);
    }
    set normalBackground(v) {
        this.i.hx = stringToBrush(v);
    }
    /**
     * Gets or sets the activation border color to use.
    */
    get activationBorder() {
        return brushToString(this.i.ht);
    }
    set activationBorder(v) {
        this.i.ht = stringToBrush(v);
    }
    /**
     * Gets or sets the amount of left activation border to use for the cell content for this column.
    */
    get activationBorderLeftWidth() {
        return this.i.dn;
    }
    set activationBorderLeftWidth(v) {
        this.i.dn = +v;
    }
    /**
     * Gets or sets the amount of top activation border to use for the cell content for this column.
    */
    get activationBorderTopWidth() {
        return this.i.dq;
    }
    set activationBorderTopWidth(v) {
        this.i.dq = +v;
    }
    /**
     * Gets or sets the amount of right activation border to use for the cell content of this column.
    */
    get activationBorderRightWidth() {
        return this.i.dp;
    }
    set activationBorderRightWidth(v) {
        this.i.dp = +v;
    }
    /**
     * Gets or sets the amount of bottom activation border to use for the cell content of this column.
    */
    get activationBorderBottomWidth() {
        return this.i.dm;
    }
    set activationBorderBottomWidth(v) {
        this.i.dm = +v;
    }
    /**
     * Sets or gets the grid selection mode.
    */
    get activationMode() {
        return this.i.r;
    }
    set activationMode(v) {
        this.i.r = ensureEnum(ListPanelActivationMode_$type, v);
    }
    /**
     * Sets or gets how selection behaves in the grid.
    */
    get selectionBehavior() {
        return this.i.al;
    }
    set selectionBehavior(v) {
        this.i.al = ensureEnum(ListPanelSelectionBehavior_$type, v);
    }
    /**
     * Sets or gets the user provided primary key to assume for the data. Certain functionality of the grid requires a way to uniquely identify items in order to work correctly.
     * If a primary key cannot be inferred from the data source, you may need to provide one here.
    */
    get primaryKey() {
        return this.i.f;
    }
    set primaryKey(v) {
        if (v && !Array.isArray(v) && typeof (v) == "string") {
            const re = /\s*(?:,|\s|$)\s*/gm;
            v = v.split(re);
        }
        this.i.f = v;
    }
    /**
     * Gets either the inferred primary key from the data source, or the user provided primary key, if present. Represents the actual primary key that the grid will use in operations that require it.
    */
    get actualPrimaryKey() {
        return this.i.d;
    }
    set actualPrimaryKey(v) {
        if (v && !Array.isArray(v) && typeof (v) == "string") {
            const re = /\s*(?:,|\s|$)\s*/gm;
            v = v.split(re);
        }
        this.i.d = v;
    }
    /**
     * Gets which items are currently selected in the grid. Collection can be updated to programatically select items.
    */
    get selectedItems() {
        if (this._selectedItems === null) {
            let coll = new IgrListPanelSelectedItemsCollection();
            let innerColl = this.i.ag;
            if (!innerColl) {
                innerColl = new ListPanelSelectedItemsCollection_internal();
            }
            this._selectedItems = coll._fromInner(innerColl);
        }
        return this._selectedItems;
    }
    set selectedItems(v) {
        if (this._selectedItems !== null) {
            this._selectedItems._setSyncTarget(null);
            this._selectedItems = null;
        }
        let coll = new IgrListPanelSelectedItemsCollection();
        this._selectedItems = coll._fromOuter(v);
        let syncColl = new SyncableObservableCollection$1(Base.$type);
        let innerColl = this.i.ag;
        if (!innerColl) {
            innerColl = new ListPanelSelectedItemsCollection_internal();
        }
        syncColl._inner = innerColl;
        syncColl.clear();
        this._selectedItems._setSyncTarget(syncColl);
    }
    /**
     * Gets which primary key values are currently selected in the grid. Collection can be updated to programatically select items.
    */
    get selectedKeys() {
        if (this._selectedKeys === null) {
            let coll = new IgrListPanelSelectedKeysCollection();
            let innerColl = this.i.aj;
            if (!innerColl) {
                innerColl = new ListPanelSelectedKeysCollection_internal();
            }
            this._selectedKeys = coll._fromInner(innerColl);
        }
        return this._selectedKeys;
    }
    set selectedKeys(v) {
        if (this._selectedKeys !== null) {
            this._selectedKeys._setSyncTarget(null);
            this._selectedKeys = null;
        }
        let coll = new IgrListPanelSelectedKeysCollection();
        this._selectedKeys = coll._fromOuter(v);
        let syncColl = new SyncableObservableCollection$1(ListPanelPrimaryKeyValue.$type);
        let innerColl = this.i.aj;
        if (!innerColl) {
            innerColl = new ListPanelSelectedKeysCollection_internal();
        }
        syncColl._inner = innerColl;
        syncColl.clear();
        this._selectedKeys._setSyncTarget(syncColl);
    }
    /**
     * Gets either the filtered properties to include in the schema of the data source.
    */
    get schemaIncludedProperties() {
        return this.i.g;
    }
    set schemaIncludedProperties(v) {
        if (v && !Array.isArray(v) && typeof (v) == "string") {
            const re = /\s*(?:,|\s|$)\s*/gm;
            v = v.split(re);
        }
        this.i.g = v;
    }
    /**
     * Gets or Sets the property name that contains the values.
    */
    get rowHeight() {
        return this.i.ea;
    }
    set rowHeight(v) {
        this.i.ea = +v;
    }
    get actualRowHeight() {
        return this.i.c2;
    }
    set actualRowHeight(v) {
        this.i.c2 = +v;
    }
    /**
     * Gets or Sets the property name that contains the values.
    */
    get cellTextStyle() {
        if (this.i.a2 == null) {
            return null;
        }
        return this.i.a2.fontString;
    }
    set cellTextStyle(v) {
        let fi = new FontInfo();
        fi.fontString = v;
        this.i.a2 = fi;
    }
    get hasUnevenSizes() {
        return this.i.bt;
    }
    /**
     * Gets or sets the style of scrollbar.
    */
    get scrollbarStyle() {
        return this.i.ay;
    }
    set scrollbarStyle(v) {
        this.i.ay = ensureEnum(ScrollbarStyle_$type, v);
    }
    /**
     * Gets or sets the scrollbar background color.
    */
    get scrollbarBackground() {
        return brushToString(this.i.hy);
    }
    set scrollbarBackground(v) {
        this.i.hy = stringToBrush(v);
    }
    /**
     * Gets or sets the color to use for the background of the component.
    */
    get backgroundColor() {
        return brushToString(this.i.hu);
    }
    set backgroundColor(v) {
        this.i.hu = stringToBrush(v);
    }
    /**
     * Gets or sets the color to use for the background of the component.
    */
    get orientation() {
        return this.i.ab;
    }
    set orientation(v) {
        this.i.ab = ensureEnum(ListPanelOrientation_$type, v);
    }
    /**
     * Gets or sets the color to use for the background of the component.
    */
    get isCustomRowHeightEnabled() {
        return this.i.bx;
    }
    set isCustomRowHeightEnabled(v) {
        this.i.bx = ensureBool(v);
    }
    /**
     * Gets or sets the color to use for the text of the component.
    */
    get textColor() {
        return brushToString(this.i.h0);
    }
    set textColor(v) {
        this.i.h0 = stringToBrush(v);
    }
    /**
     * Gets or sets the position of the active row.
    */
    get activeRow() {
        return this.i.dr;
    }
    set activeRow(v) {
        this.i.dr = +v;
    }
    /**
     * Gets or sets if the grid should notify on programmatic selection changes as well as changes from the UI.
    */
    get notifyOnAllSelectionChanges() {
        return this.i.b0;
    }
    set notifyOnAllSelectionChanges(v) {
        this.i.b0 = ensureBool(v);
    }
    findByName(name) {
        if (this.findEphemera) {
            if (name && name.indexOf("@@e:") == 0) {
                return this.findEphemera(name);
            }
        }
        if (this.selectedKeys != null && this.selectedKeys.findByName && this.selectedKeys.findByName(name)) {
            return this.selectedKeys.findByName(name);
        }
        return null;
    }
    get hasUserValues() {
        return this._hasUserValues;
    }
    __m(propertyName) {
        if (!this._inStyling) {
            this._hasUserValues.add(propertyName);
        }
    }
    _styling(container, component, parent) {
        if (this._inStyling) {
            return;
        }
        this._inStyling = true;
        this._stylingContainer = container;
        this._stylingParent = component;
        let genericPrefix = "";
        let typeName = this.i.$type.name;
        if (typeName.indexOf("Xam") === 0) {
            typeName = typeName.substring(3);
        }
        genericPrefix = toSpinal("ListPanel");
        let additionalPrefixes = [];
        let prefix = toSpinal(typeName);
        additionalPrefixes.push(prefix + "-");
        let b = this.i.$type.baseType;
        while (b && b.name != "Object" &&
            b.name != "Base" &&
            b.name != "Control" &&
            b.Name != "DependencyObject" &&
            b.Name != "FrameworkElement") {
            typeName = b.name;
            if (typeName.indexOf("Xam") === 0) {
                typeName = typeName.substring(3);
            }
            let basePrefix = toSpinal(typeName);
            additionalPrefixes.push(basePrefix + "-");
            b = b.baseType;
        }
        if (parent) {
            let parentTypeName = parent.i.$type.name;
            if (parentTypeName.indexOf("Xam") === 0) {
                parentTypeName = parentTypeName.substring(3);
            }
            let parentPrefix = toSpinal(parentTypeName);
            additionalPrefixes.push(parentPrefix + "-" + genericPrefix + "-");
            additionalPrefixes.push(parentPrefix + "-" + prefix + "-");
        }
        initializePropertiesFromCss(container, this, genericPrefix + "-", this.hasUserValues, false, additionalPrefixes);
        if (this._otherStyling) {
            this._otherStyling(container, component, parent);
        }
        this._inStyling = false;
    }
    getRowKey(primaryKey, rowIndex) {
        let iv = this.i.ad(primaryKey, rowIndex);
        let ret = null;
        if (iv && iv.externalObject) {
            ret = iv.externalObject;
        }
        else {
            if (iv) {
                let e = new IgrListPanelPrimaryKeyValue();
                e._implementation = iv;
                iv.externalObject = e;
                ret = e;
            }
        }
        return ret;
    }
    getItemKey(primaryKey, item_) {
        let iv = this.i.ac(primaryKey, item_);
        let ret = null;
        if (iv && iv.externalObject) {
            ret = iv.externalObject;
        }
        else {
            if (iv) {
                let e = new IgrListPanelPrimaryKeyValue();
                e._implementation = iv;
                iv.externalObject = e;
                ret = e;
            }
        }
        return ret;
    }
    /**
     * Gets the last visible index in the grid.
    
    */
    getLastVisibleIndex() {
        let iv = this.i.d0();
        return (iv);
    }
    /**
     * Gets the first visible index in the grid.
    
    */
    getFirstVisibleIndex() {
        let iv = this.i.dy();
        return (iv);
    }
    /**
     * Invalidates the bound data for the rows currently visible.
    
    */
    invalidateVisibleItems() {
        this.i.f0();
    }
    notifySetItem(index, oldItem, newItem) {
        this.i.f6(index, oldItem, newItem);
    }
    /**
     * Manually notifies the grid's data source that the data it has bound to has been cleared and needs to be re-examined.
     * This should not be called if the data that the grid is bound to is already observable.
    
    */
    notifyClearItems() {
        this.i.f3();
    }
    notifyInsertItem(index, newItem) {
        this.i.f4(index, newItem);
    }
    notifyRemoveItem(index, oldItem) {
        this.i.f5(index, oldItem);
    }
    /**
     * Returns the data index of an item within the bound data source.
    
    * @param item  * The item from which to get the index.
    */
    dataIndexOfItem(item) {
        let iv = this.i.dt(item);
        return (iv);
    }
    dataIndexOfPrimaryKey(key) {
        let iv = this.i.du(key);
        return (iv);
    }
    exportVisualData() {
        let iv = this.i.ed();
        return (iv);
    }
    exportSerializedVisualData() {
        let iv = this.i.ev();
        return (iv);
    }
    /**
     * Selects all of the rows in the grid.
    
    */
    selectAllRows() {
        this.i.hc();
    }
    /**
     * Scrolls the grid to given row, by index
    
    * @param rowNumber  * The row index to scroll to.
    */
    scrollToRowByIndex(rowNumber) {
        this.i.ha(rowNumber);
    }
    /**
     * Scrolls the grid so that a row is the last visible, by index.
    
    * @param rowNumber  * The row index to scroll to.
    */
    scrollToLastRowByIndex(rowNumber) {
        this.i.g8(rowNumber);
    }
    /**
     * Deselects all currently selected rows.
    
    */
    deselectAllRows() {
        this.i.fk();
    }
    /**
     * Called when the selected items change.
    */
    get selectedItemsChanged() {
        return this._selectedItemsChanged;
    }
    set selectedItemsChanged(ev) {
        if (this._selectedItemsChanged_wrapped !== null) {
            this.i.selectedItemsChanged = delegateRemove(this.i.selectedItemsChanged, this._selectedItemsChanged_wrapped);
            this._selectedItemsChanged_wrapped = null;
            this._selectedItemsChanged = null;
        }
        this._selectedItemsChanged = ev;
        this._selectedItemsChanged_wrapped = (o, e) => {
            let outerArgs = new IgrListPanelSelectedItemsChangedEventArgs();
            outerArgs._provideImplementation(e);
            if (this.beforeSelectedItemsChanged) {
                this.beforeSelectedItemsChanged(this, outerArgs);
            }
            if (this._selectedItemsChanged) {
                this._selectedItemsChanged(this, outerArgs);
            }
        };
        this.i.selectedItemsChanged = delegateCombine(this.i.selectedItemsChanged, this._selectedItemsChanged_wrapped);
        ;
    }
    /**
     * Called when the selected keys change.
    */
    get selectedKeysChanged() {
        return this._selectedKeysChanged;
    }
    set selectedKeysChanged(ev) {
        if (this._selectedKeysChanged_wrapped !== null) {
            this.i.selectedKeysChanged = delegateRemove(this.i.selectedKeysChanged, this._selectedKeysChanged_wrapped);
            this._selectedKeysChanged_wrapped = null;
            this._selectedKeysChanged = null;
        }
        this._selectedKeysChanged = ev;
        this._selectedKeysChanged_wrapped = (o, e) => {
            let outerArgs = new IgrListPanelSelectedKeysChangedEventArgs();
            outerArgs._provideImplementation(e);
            if (this.beforeSelectedKeysChanged) {
                this.beforeSelectedKeysChanged(this, outerArgs);
            }
            if (this._selectedKeysChanged) {
                this._selectedKeysChanged(this, outerArgs);
            }
        };
        this.i.selectedKeysChanged = delegateCombine(this.i.selectedKeysChanged, this._selectedKeysChanged_wrapped);
        ;
    }
    /**
     * Called when an individual selection operation has been made.
    */
    get selectionChanged() {
        return this._selectionChanged;
    }
    set selectionChanged(ev) {
        if (this._selectionChanged_wrapped !== null) {
            this.i.selectionChanged = delegateRemove(this.i.selectionChanged, this._selectionChanged_wrapped);
            this._selectionChanged_wrapped = null;
            this._selectionChanged = null;
        }
        this._selectionChanged = ev;
        this._selectionChanged_wrapped = (o, e) => {
            let outerArgs = new IgrListPanelSelectionChangedEventArgs();
            outerArgs._provideImplementation(e);
            if (this.beforeSelectionChanged) {
                this.beforeSelectionChanged(this, outerArgs);
            }
            if (this._selectionChanged) {
                this._selectionChanged(this, outerArgs);
            }
        };
        this.i.selectionChanged = delegateCombine(this.i.selectionChanged, this._selectionChanged_wrapped);
        ;
    }
    /**
     * Called when the active row changes.
    */
    get activeRowChanged() {
        return this._activeRowChanged;
    }
    set activeRowChanged(ev) {
        if (this._activeRowChanged_wrapped !== null) {
            this.i.activeRowChanged = delegateRemove(this.i.activeRowChanged, this._activeRowChanged_wrapped);
            this._activeRowChanged_wrapped = null;
            this._activeRowChanged = null;
        }
        this._activeRowChanged = ev;
        this._activeRowChanged_wrapped = (o, e) => {
            let outerArgs = new IgrListPanelActiveRowChangedEventArgs();
            outerArgs._provideImplementation(e);
            if (this.beforeActiveRowChanged) {
                this.beforeActiveRowChanged(this, outerArgs);
            }
            if (this._activeRowChanged) {
                this._activeRowChanged(this, outerArgs);
            }
        };
        this.i.activeRowChanged = delegateCombine(this.i.activeRowChanged, this._activeRowChanged_wrapped);
        ;
    }
    get contentRefreshed() {
        return this._contentRefreshed;
    }
    set contentRefreshed(ev) {
        if (this._contentRefreshed_wrapped !== null) {
            this.i.contentRefreshed = delegateRemove(this.i.contentRefreshed, this._contentRefreshed_wrapped);
            this._contentRefreshed_wrapped = null;
            this._contentRefreshed = null;
        }
        this._contentRefreshed = ev;
        this._contentRefreshed_wrapped = (o, e) => {
            let outerArgs = new IgrListPanelContentRefreshedEventArgs();
            outerArgs._provideImplementation(e);
            if (this.beforeContentRefreshed) {
                this.beforeContentRefreshed(this, outerArgs);
            }
            if (this._contentRefreshed) {
                this._contentRefreshed(this, outerArgs);
            }
        };
        this.i.contentRefreshed = delegateCombine(this.i.contentRefreshed, this._contentRefreshed_wrapped);
        ;
    }
    get rowUpdating() {
        return this._rowUpdating;
    }
    set rowUpdating(ev) {
        if (this._rowUpdating_wrapped !== null) {
            this.i.rowUpdating = delegateRemove(this.i.rowUpdating, this._rowUpdating_wrapped);
            this._rowUpdating_wrapped = null;
            this._rowUpdating = null;
        }
        this._rowUpdating = ev;
        this._rowUpdating_wrapped = (o, e) => {
            let outerArgs = new IgrListPanelTemplateItemUpdatingEventArgs();
            outerArgs._provideImplementation(e);
            if (this.beforeRowUpdating) {
                this.beforeRowUpdating(this, outerArgs);
            }
            if (this._rowUpdating) {
                this._rowUpdating(this, outerArgs);
            }
        };
        this.i.rowUpdating = delegateCombine(this.i.rowUpdating, this._rowUpdating_wrapped);
        ;
    }
    get itemWidthRequested() {
        return this._itemWidthRequested;
    }
    set itemWidthRequested(ev) {
        if (this._itemWidthRequested_wrapped !== null) {
            this.i.itemWidthRequested = delegateRemove(this.i.itemWidthRequested, this._itemWidthRequested_wrapped);
            this._itemWidthRequested_wrapped = null;
            this._itemWidthRequested = null;
        }
        this._itemWidthRequested = ev;
        this._itemWidthRequested_wrapped = (o, e) => {
            let outerArgs = new IgrListPanelTemplateWidthRequestedEventArgs();
            outerArgs._provideImplementation(e);
            if (this.beforeItemWidthRequested) {
                this.beforeItemWidthRequested(this, outerArgs);
            }
            if (this._itemWidthRequested) {
                this._itemWidthRequested(this, outerArgs);
            }
        };
        this.i.itemWidthRequested = delegateCombine(this.i.itemWidthRequested, this._itemWidthRequested_wrapped);
        ;
    }
    get itemHeightRequested() {
        return this._itemHeightRequested;
    }
    set itemHeightRequested(ev) {
        if (this._itemHeightRequested_wrapped !== null) {
            this.i.itemHeightRequested = delegateRemove(this.i.itemHeightRequested, this._itemHeightRequested_wrapped);
            this._itemHeightRequested_wrapped = null;
            this._itemHeightRequested = null;
        }
        this._itemHeightRequested = ev;
        this._itemHeightRequested_wrapped = (o, e) => {
            let outerArgs = new IgrListPanelTemplateHeightRequestedEventArgs();
            outerArgs._provideImplementation(e);
            if (this.beforeItemHeightRequested) {
                this.beforeItemHeightRequested(this, outerArgs);
            }
            if (this._itemHeightRequested) {
                this._itemHeightRequested(this, outerArgs);
            }
        };
        this.i.itemHeightRequested = delegateCombine(this.i.itemHeightRequested, this._itemHeightRequested_wrapped);
        ;
    }
    get itemClicked() {
        return this._itemClicked;
    }
    set itemClicked(ev) {
        if (this._itemClicked_wrapped !== null) {
            this.i.itemClicked = delegateRemove(this.i.itemClicked, this._itemClicked_wrapped);
            this._itemClicked_wrapped = null;
            this._itemClicked = null;
        }
        this._itemClicked = ev;
        this._itemClicked_wrapped = (o, e) => {
            let outerArgs = new IgrListPanelItemEventArgs();
            outerArgs._provideImplementation(e);
            if (this.beforeItemClicked) {
                this.beforeItemClicked(this, outerArgs);
            }
            if (this._itemClicked) {
                this._itemClicked(this, outerArgs);
            }
        };
        this.i.itemClicked = delegateCombine(this.i.itemClicked, this._itemClicked_wrapped);
        ;
    }
    get actualPrimaryKeyChange() {
        return this._actualPrimaryKeyChange;
    }
    set actualPrimaryKeyChange(ev) {
        if (this._actualPrimaryKeyChange_wrapped !== null) {
            this.i.propertyChanged = delegateRemove(this.i.propertyChanged, this._actualPrimaryKeyChange_wrapped);
            this._actualPrimaryKeyChange_wrapped = null;
            this._actualPrimaryKeyChange = null;
        }
        this._actualPrimaryKeyChange = ev;
        this._actualPrimaryKeyChange_wrapped = (o, e) => {
            let ext = this.actualPrimaryKey;
            if (this.beforeActualPrimaryKeyChange) {
                this.beforeActualPrimaryKeyChange(this, ext);
            }
            if (this._actualPrimaryKeyChange) {
                this._actualPrimaryKeyChange(this, ext);
            }
        };
        this.i.propertyChanged = delegateCombine(this.i.propertyChanged, this._actualPrimaryKeyChange_wrapped);
    }
}
