import { IgrToolAction } from "./igr-tool-action";
import { ToolActionRadio } from "./ToolActionRadio";
import { ensureBool, brushToString, stringToBrush } from "igniteui-react-core";
export class IgrToolActionRadio extends IgrToolAction {
    createImplementation() {
        return new ToolActionRadio();
    }
    /**
                                 * @hidden
                                 */
    get i() {
        return this._implementation;
    }
    constructor(props) {
        super(props);
    }
    /**
     * Gets or sets the channel this radio button belongs to.
    */
    get channel() {
        return this.i.hd;
    }
    set channel(v) {
        this.i.hd = v;
    }
    /**
     * Gets or sets if the tool action is checked.
    */
    get isChecked() {
        return this.i.g6;
    }
    set isChecked(v) {
        this.i.g6 = ensureBool(v);
    }
    /**
     * Gets or sets if the checked state of the radio is handled manually by the user. The radio
     * well not become selected when clicked if this property is true.
    */
    get isManual() {
        return this.i.g7;
    }
    set isManual(v) {
        this.i.g7 = ensureBool(v);
    }
    /**
     * Gets or sets the background color of the radio when in the unchecked state.
    */
    get uncheckedBackgroundColor() {
        return brushToString(this.i.ht);
    }
    set uncheckedBackgroundColor(v) {
        this.i.ht = stringToBrush(v);
    }
    /**
     * Gets the actual background color of the radio when in the unchecked state.
    */
    get actualUncheckedBackgroundColor() {
        return brushToString(this.i.hp);
    }
    set actualUncheckedBackgroundColor(v) {
        this.i.hp = stringToBrush(v);
    }
    /**
     * Gets or sets the background color of the radio when in the checked state.
    */
    get checkedBackgroundColor() {
        return brushToString(this.i.hr);
    }
    set checkedBackgroundColor(v) {
        this.i.hr = stringToBrush(v);
    }
    /**
     * Gets the actual background color of the radio when in the checked state.
    */
    get actualCheckedBackgroundColor() {
        return brushToString(this.i.hn);
    }
    set actualCheckedBackgroundColor(v) {
        this.i.hn = stringToBrush(v);
    }
    /**
     * Gets or sets the border color of the radio when in the unchecked state.
    */
    get uncheckedBorderColor() {
        return brushToString(this.i.hu);
    }
    set uncheckedBorderColor(v) {
        this.i.hu = stringToBrush(v);
    }
    /**
     * Gets the actual border color of the radio when in the unchecked state.
    */
    get actualUncheckedBorderColor() {
        return brushToString(this.i.hq);
    }
    set actualUncheckedBorderColor(v) {
        this.i.hq = stringToBrush(v);
    }
    /**
     * Gets or sets the border color of the radio when in the checked state.
    */
    get checkedBorderColor() {
        return brushToString(this.i.hs);
    }
    set checkedBorderColor(v) {
        this.i.hs = stringToBrush(v);
    }
    /**
     * Gets the actual border color of the radio when in the checked state.
    */
    get actualCheckedBorderColor() {
        return brushToString(this.i.ho);
    }
    set actualCheckedBorderColor(v) {
        this.i.ho = stringToBrush(v);
    }
}
