import * as React from 'react';
import { delegateCombine, delegateRemove } from "igniteui-react-core";
import { IgrToolContextBindingCollection } from "./igr-tool-context-binding-collection";
import { IgrToolActionCollection } from "./igr-tool-action-collection";
import { ControlDisplayDensity_$type } from "igniteui-react-core";
import { Visibility_$type } from "igniteui-react-core";
import { IgrToolCommandEventArgs } from "./igr-tool-command-event-args";
import { IgrToolActionPerformedEventArgs } from "./igr-tool-action-performed-event-args";
import { ToolAction } from "./ToolAction";
import { TypeRegistrar } from "igniteui-react-core";
import { NamePatcher, getModifiedProps, isValidProp, ensureBool, brushToString, stringToBrush, ensureEnum, toBrushCollection, fromBrushCollection, toSpinal, initializePropertiesFromCss } from "igniteui-react-core";
import { ToolContextBindingCollection as ToolContextBindingCollection_internal } from "./ToolContextBindingCollection";
import { ToolContextBinding } from "./ToolContextBinding";
import { SyncableObservableCollection$1 } from "igniteui-react-core";
import { ToolActionCollection as ToolActionCollection_internal } from "./ToolActionCollection";
import { FontInfo } from "igniteui-react-core";
/**
 * Represents base class for all tool actions in toolbar
*/
export class IgrToolAction extends React.Component {
    createImplementation() {
        return null;
    }
    get nativeElement() {
        return this._implementation.nativeElement;
    }
    /**
     * @hidden
     */
    get i() {
        return this._implementation;
    } /**
                                 * @hidden
                                 */
    static _createFromInternal(internal) {
        if (!internal) {
            return null;
        }
        if (!internal.$type) {
            return null;
        }
        let name = internal.$type.name;
        let externalName = "Igr" + name;
        if (!TypeRegistrar.isRegistered(externalName)) {
            return null;
        }
        return TypeRegistrar.create(externalName);
    }
    onImplementationCreated() {
    }
    constructor(props) {
        super(props);
        this.mounted = false;
        this._contextBindings = null;
        this._actions = null;
        this.__p = null;
        this._hasUserValues = new Set();
        this._stylingContainer = null;
        this._stylingParent = null;
        this._inStyling = false;
        this._onCommand = null;
        this._onCommand_wrapped = null;
        this._performed = null;
        this._performed_wrapped = null;
        if (this._styling) {
            NamePatcher.ensureStylablePatched(Object.getPrototypeOf(this));
        }
        this._implementation = this.createImplementation();
        this._implementation.externalObject = this;
        this.onImplementationCreated();
        if (this._initializeAdapters) {
            this._initializeAdapters();
        }
    }
    componentDidMount() {
        this.mounted = true;
        for (const p of Object.keys(this.props)) {
            if (isValidProp(this, p)) {
                this[p] = this.props[p];
            }
        }
    }
    shouldComponentUpdate(nextProps, nextState) {
        const mod = getModifiedProps(this.props, nextProps);
        for (const p of Object.keys(mod)) {
            if (isValidProp(this, p)) {
                this[p] = mod[p];
            }
        }
        return true;
    }
    render() {
        return null;
    }
    get isOpen() {
        return this.i.a7;
    }
    /**
     * Gets the context bindings for this tool action.
    */
    get contextBindings() {
        if (this._contextBindings === null) {
            let coll = new IgrToolContextBindingCollection();
            let innerColl = this.i.contextBindings;
            if (!innerColl) {
                innerColl = new ToolContextBindingCollection_internal();
            }
            this._contextBindings = coll._fromInner(innerColl);
            this.i.contextBindings = innerColl;
        }
        return this._contextBindings;
    }
    set contextBindings(v) {
        if (this._contextBindings !== null) {
            this._contextBindings._setSyncTarget(null);
            this._contextBindings = null;
        }
        let coll = new IgrToolContextBindingCollection();
        this._contextBindings = coll._fromOuter(v);
        let syncColl = new SyncableObservableCollection$1(ToolContextBinding.$type);
        let innerColl = this.i.contextBindings;
        if (!innerColl) {
            innerColl = new ToolContextBindingCollection_internal();
        }
        syncColl._inner = innerColl;
        syncColl.clear();
        this._contextBindings._setSyncTarget(syncColl);
        this.i.contextBindings = innerColl;
    }
    get actionId() {
        return this.i.cs;
    }
    /**
     * Gets the sub actions associated with this tool action.
    */
    get actions() {
        if (this._actions === null) {
            let coll = new IgrToolActionCollection();
            let innerColl = this.i.actions;
            if (!innerColl) {
                innerColl = new ToolActionCollection_internal();
            }
            this._actions = coll._fromInner(innerColl);
        }
        return this._actions;
    }
    set actions(v) {
        if (this._actions !== null) {
            this._actions._setSyncTarget(null);
            this._actions = null;
        }
        let coll = new IgrToolActionCollection();
        this._actions = coll._fromOuter(v);
        let syncColl = new SyncableObservableCollection$1(ToolAction.$type);
        let innerColl = this.i.actions;
        if (!innerColl) {
            innerColl = new ToolActionCollection_internal();
        }
        syncColl._inner = innerColl;
        syncColl.clear();
        this._actions._setSyncTarget(syncColl);
    }
    /**
     * Gets or sets the width to use for this tool action.
    */
    get width() {
        return this.i.ci;
    }
    set width(v) {
        this.i.ci = +v;
    }
    get parentId() {
        return this.i.ek;
    }
    set parentId(v) {
        this.i.ek = v;
    }
    /**
     * Gets or sets the Id of the action that this action will be inserted before.
     * Subsequent actions will continue to insert before the target action, following
     * the original action that set this property. If using this property do not use the
     * AfterId property.
     * This property is only relevant during initial insertion into the toolbar. Changing the property
     * after the action has already been added will have no effect.
    */
    get beforeId() {
        return this.i.df;
    }
    set beforeId(v) {
        this.i.df = v;
    }
    /**
     * Gets or sets the Id of the action that this action will be inserted after.
     * Subsequent actions will continue to insert after the target action, following
     * the original action that set this property. If using this property do not use the
     * BeforeId property.
     * This property is only relevant during initial insertion into the toolbar. Changing the property
     * after the action has already been added will have no effect.
    */
    get afterId() {
        return this.i.dc;
    }
    set afterId(v) {
        this.i.dc = v;
    }
    /**
     * Sets to the Id of the action that this action will overlay itself on to.
     * The purpose of this property is to allow users to override actions automatically generated by
     * toolbar targets such as the DataChart. These generated actions all have unique Ids associated
     * with them that can be used for the OverlayId.
     * These overlaid actions will
     * not
     * replace the generated one but will have its properties
     * overlaid onto the generated actions.
    */
    get overlayId() {
        return this.i.ef;
    }
    set overlayId(v) {
        this.i.ef = v;
    }
    get commandId() {
        return this.i.dk;
    }
    set commandId(v) {
        this.i.dk = v;
    }
    get commandArgument() {
        return this.i.dh;
    }
    set commandArgument(v) {
        this.i.dh = v;
    }
    get commandArgumentValue() {
        return this.i.cj;
    }
    set commandArgumentValue(v) {
        this.i.cj = v;
    }
    get paddingLeft() {
        return this.i.ce;
    }
    set paddingLeft(v) {
        this.i.ce = +v;
    }
    get actualPaddingLeft() {
        return this.i.bw;
    }
    set actualPaddingLeft(v) {
        this.i.bw = +v;
    }
    get paddingRight() {
        return this.i.cf;
    }
    set paddingRight(v) {
        this.i.cf = +v;
    }
    get actualPaddingRight() {
        return this.i.bx;
    }
    set actualPaddingRight(v) {
        this.i.bx = +v;
    }
    get paddingBottom() {
        return this.i.cd;
    }
    set paddingBottom(v) {
        this.i.cd = +v;
    }
    get actualPaddingBottom() {
        return this.i.bv;
    }
    set actualPaddingBottom(v) {
        this.i.bv = +v;
    }
    get paddingTop() {
        return this.i.cg;
    }
    set paddingTop(v) {
        this.i.cg = +v;
    }
    get actualPaddingTop() {
        return this.i.by;
    }
    set actualPaddingTop(v) {
        this.i.by = +v;
    }
    /**
     * Gets the actual hover background used by the tool action.
    */
    get actualHoverBackground() {
        return brushToString(this.i.gr);
    }
    set actualHoverBackground(v) {
        this.i.gr = stringToBrush(v);
    }
    /**
     * Gets or sets the hover background for this tool action.
    */
    get hoverBackground() {
        return brushToString(this.i.gz);
    }
    set hoverBackground(v) {
        this.i.gz = stringToBrush(v);
    }
    /**
     * Gets the actual background used by the tool action.
    */
    get actualBackground() {
        return brushToString(this.i.go);
    }
    set actualBackground(v) {
        this.i.go = stringToBrush(v);
    }
    /**
     * Gets or sets the background color of the tool action.
    */
    get background() {
        return brushToString(this.i.gw);
    }
    set background(v) {
        this.i.gw = stringToBrush(v);
    }
    /**
     * Gets the actual label text color.
    */
    get actualTextColor() {
        return brushToString(this.i.gv);
    }
    set actualTextColor(v) {
        this.i.gv = stringToBrush(v);
    }
    /**
     * Gets or sets the label text color.
    */
    get textColor() {
        return brushToString(this.i.g3);
    }
    set textColor(v) {
        this.i.g3 = stringToBrush(v);
    }
    /**
     * Gets the actual label text color.
    */
    get actualSubtitleTextColor() {
        return brushToString(this.i.gu);
    }
    set actualSubtitleTextColor(v) {
        this.i.gu = stringToBrush(v);
    }
    /**
     * Gets or sets the label text color.
    */
    get subtitleTextColor() {
        return brushToString(this.i.g2);
    }
    set subtitleTextColor(v) {
        this.i.g2 = stringToBrush(v);
    }
    /**
     * Gets the actual label text color when the tool is disabled.
    */
    get actualDisabledTextColor() {
        return brushToString(this.i.gp);
    }
    set actualDisabledTextColor(v) {
        this.i.gp = stringToBrush(v);
    }
    /**
     * Gets or sets the label text color when the tool is disabled.
    */
    get disabledTextColor() {
        return brushToString(this.i.gx);
    }
    set disabledTextColor(v) {
        this.i.gx = stringToBrush(v);
    }
    /**
     * Gets the actual font used.
    */
    get actualTextStyle() {
        return this.i.ad;
    }
    set actualTextStyle(v) {
        this.i.ad = v;
    }
    /**
     * Gets or sets the font to use.
    */
    get textStyle() {
        if (this.i.ag == null) {
            return null;
        }
        return this.i.ag.fontString;
    }
    set textStyle(v) {
        let fi = new FontInfo();
        fi.fontString = v;
        this.i.ag = fi;
    }
    /**
     * Gets the actual font used.
    */
    get actualSubtitleTextStyle() {
        return this.i.ac;
    }
    set actualSubtitleTextStyle(v) {
        this.i.ac = v;
    }
    /**
     * Gets or sets the font to use.
    */
    get subtitleTextStyle() {
        if (this.i.af == null) {
            return null;
        }
        return this.i.af.fontString;
    }
    set subtitleTextStyle(v) {
        let fi = new FontInfo();
        fi.fontString = v;
        this.i.af = fi;
    }
    get isDisabled() {
        return this.i.a5;
    }
    set isDisabled(v) {
        this.i.a5 = ensureBool(v);
    }
    get isHighlighted() {
        return this.i.a6;
    }
    set isHighlighted(v) {
        this.i.a6 = ensureBool(v);
    }
    /**
     * Gets or sets the display density to use for the action.
    */
    get density() {
        return this.i.v;
    }
    set density(v) {
        this.i.v = ensureEnum(ControlDisplayDensity_$type, v);
    }
    /**
     * Gets the resolved display density to use for the action.
    */
    get actualDensity() {
        return this.i.u;
    }
    set actualDensity(v) {
        this.i.u = ensureEnum(ControlDisplayDensity_$type, v);
    }
    get name() {
        return this.i.ed;
    }
    set name(v) {
        this.i.ed = v;
    }
    /**
     * Gets or sets the title of the tool.
    */
    get title() {
        return this.i.ev;
    }
    set title(v) {
        this.i.ev = v;
    }
    /**
     * Gets or sets the subtitle of the tool.
    */
    get subtitle() {
        return this.i.ep;
    }
    set subtitle(v) {
        this.i.ep = v;
    }
    get iconName() {
        return this.i.d1;
    }
    set iconName(v) {
        this.i.d1 = v;
    }
    get iconCollectionName() {
        return this.i.dw;
    }
    set iconCollectionName(v) {
        this.i.dw = v;
    }
    /**
     * Gets or sets the icon viewbox left.
    */
    get iconViewBoxLeft() {
        return this.i.b9;
    }
    set iconViewBoxLeft(v) {
        this.i.b9 = +v;
    }
    /**
     * Gets or sets the icon viewbox top.
    */
    get iconViewBoxTop() {
        return this.i.ca;
    }
    set iconViewBoxTop(v) {
        this.i.ca = +v;
    }
    /**
     * Gets or sets the icon viewbox width.
    */
    get iconViewBoxWidth() {
        return this.i.cb;
    }
    set iconViewBoxWidth(v) {
        this.i.cb = +v;
    }
    /**
     * Gets or sets the icon viewbox height.
    */
    get iconViewBoxHeight() {
        return this.i.b8;
    }
    set iconViewBoxHeight(v) {
        this.i.b8 = +v;
    }
    /**
     * Gets or sets the icon fill color.
    */
    get iconFill() {
        return brushToString(this.i.g0);
    }
    set iconFill(v) {
        this.i.g0 = stringToBrush(v);
    }
    /**
     * Gets the actual icon fill color.
    */
    get actualIconFill() {
        return brushToString(this.i.gs);
    }
    set actualIconFill(v) {
        this.i.gs = stringToBrush(v);
    }
    /**
     * Gets or sets the icon fill colors.
    */
    get iconFillColors() {
        return fromBrushCollection(this.i.a);
    }
    set iconFillColors(v) {
        this.i.a = toBrushCollection(v);
    }
    /**
     * Gets or sets the icon stroke color.
    */
    get iconStroke() {
        return brushToString(this.i.g1);
    }
    set iconStroke(v) {
        this.i.g1 = stringToBrush(v);
    }
    /**
     * Gets the actual icon stroke color.
    */
    get actualIconStroke() {
        return brushToString(this.i.gt);
    }
    set actualIconStroke(v) {
        this.i.gt = stringToBrush(v);
    }
    /**
     * Gets or sets the icon stroke colors.
    */
    get iconStrokeColors() {
        return fromBrushCollection(this.i.b);
    }
    set iconStrokeColors(v) {
        this.i.b = toBrushCollection(v);
    }
    /**
     * Gets or sets the icon viewbox width.
    */
    get iconStrokeWidth() {
        return this.i.b7;
    }
    set iconStrokeWidth(v) {
        this.i.b7 = +v;
    }
    /**
     * Gets or sets the width of the icon.
    */
    get iconWidth() {
        return this.i.cc;
    }
    set iconWidth(v) {
        this.i.cc = +v;
    }
    /**
     * Gets the actual icon width used.
    */
    get actualIconWidth() {
        return this.i.bu;
    }
    set actualIconWidth(v) {
        this.i.bu = +v;
    }
    /**
     * Gets or sets the height of the icon.
    */
    get iconHeight() {
        return this.i.b6;
    }
    set iconHeight(v) {
        this.i.b6 = +v;
    }
    /**
     * Gets the actual icon height used.
    */
    get actualIconHeight() {
        return this.i.bt;
    }
    set actualIconHeight(v) {
        this.i.bt = +v;
    }
    /**
     * Gets or sets the tool action visibility.
    */
    get visibility() {
        return this.i.g4;
    }
    set visibility(v) {
        this.i.g4 = ensureEnum(Visibility_$type, v);
    }
    /**
     * Gets or sets if the tool action will close its parent dropdown when executed by the user.
    */
    get closeOnExecute() {
        return this.i.at;
    }
    set closeOnExecute(v) {
        this.i.at = ensureBool(v);
    }
    get highlightWidth() {
        return this.i.b5;
    }
    set highlightWidth(v) {
        this.i.b5 = +v;
    }
    get actualHighlightWidth() {
        return this.i.bs;
    }
    set actualHighlightWidth(v) {
        this.i.bs = +v;
    }
    get highlightRadius() {
        return this.i.b4;
    }
    set highlightRadius(v) {
        this.i.b4 = +v;
    }
    get actualHighlightRadius() {
        return this.i.br;
    }
    set actualHighlightRadius(v) {
        this.i.br = +v;
    }
    get highlightColor() {
        return brushToString(this.i.gy);
    }
    set highlightColor(v) {
        this.i.gy = stringToBrush(v);
    }
    get actualHighlightColor() {
        return brushToString(this.i.gq);
    }
    set actualHighlightColor(v) {
        this.i.gq = stringToBrush(v);
    }
    findByName(name) {
        if (this.findEphemera) {
            if (name && name.indexOf("@@e:") == 0) {
                return this.findEphemera(name);
            }
        }
        if (this.contextBindings != null && this.contextBindings.findByName && this.contextBindings.findByName(name)) {
            return this.contextBindings.findByName(name);
        }
        if (this.actions != null && this.actions.findByName && this.actions.findByName(name)) {
            return this.actions.findByName(name);
        }
        return null;
    }
    get hasUserValues() {
        return this._hasUserValues;
    }
    __m(propertyName) {
        if (!this._inStyling) {
            this._hasUserValues.add(propertyName);
        }
    }
    _styling(container, component, parent) {
        if (this._inStyling) {
            return;
        }
        this._inStyling = true;
        this._stylingContainer = container;
        this._stylingParent = component;
        let genericPrefix = "";
        let typeName = this.i.$type.name;
        if (typeName.indexOf("Xam") === 0) {
            typeName = typeName.substring(3);
        }
        genericPrefix = toSpinal("ToolAction");
        let additionalPrefixes = [];
        let prefix = toSpinal(typeName);
        additionalPrefixes.push(prefix + "-");
        let b = this.i.$type.baseType;
        while (b && b.name != "Object" &&
            b.name != "Base" &&
            b.name != "Control" &&
            b.Name != "DependencyObject" &&
            b.Name != "FrameworkElement") {
            typeName = b.name;
            if (typeName.indexOf("Xam") === 0) {
                typeName = typeName.substring(3);
            }
            let basePrefix = toSpinal(typeName);
            additionalPrefixes.push(basePrefix + "-");
            b = b.baseType;
        }
        if (parent) {
            let parentTypeName = parent.i.$type.name;
            if (parentTypeName.indexOf("Xam") === 0) {
                parentTypeName = parentTypeName.substring(3);
            }
            let parentPrefix = toSpinal(parentTypeName);
            additionalPrefixes.push(parentPrefix + "-" + genericPrefix + "-");
            additionalPrefixes.push(parentPrefix + "-" + prefix + "-");
        }
        initializePropertiesFromCss(container, this, genericPrefix + "-", this.hasUserValues, false, additionalPrefixes);
        if (this._otherStyling) {
            this._otherStyling(container, component, parent);
        }
        this._inStyling = false;
    }
    cleanUp() {
        this.i.e0();
    }
    openSubMenu() {
        this.i.f0();
    }
    closeSubmenu() {
        this.i.e2();
    }
    exportVisualData() {
        let iv = this.i.ck();
        return (iv);
    }
    exportSerializedVisualData() {
        let iv = this.i.dr();
        return (iv);
    }
    get onCommand() {
        return this._onCommand;
    }
    set onCommand(ev) {
        if (this._onCommand_wrapped !== null) {
            this.i.onCommand = delegateRemove(this.i.onCommand, this._onCommand_wrapped);
            this._onCommand_wrapped = null;
            this._onCommand = null;
        }
        this._onCommand = ev;
        this._onCommand_wrapped = (o, e) => {
            let outerArgs = new IgrToolCommandEventArgs();
            outerArgs._provideImplementation(e);
            if (this.beforeOnCommand) {
                this.beforeOnCommand(this, outerArgs);
            }
            if (this._onCommand) {
                this._onCommand(this, outerArgs);
            }
        };
        this.i.onCommand = delegateCombine(this.i.onCommand, this._onCommand_wrapped);
        ;
    }
    get performed() {
        return this._performed;
    }
    set performed(ev) {
        if (this._performed_wrapped !== null) {
            this.i.performed = delegateRemove(this.i.performed, this._performed_wrapped);
            this._performed_wrapped = null;
            this._performed = null;
        }
        this._performed = ev;
        this._performed_wrapped = (o, e) => {
            let outerArgs = new IgrToolActionPerformedEventArgs();
            outerArgs._provideImplementation(e);
            if (this.beforePerformed) {
                this.beforePerformed(this, outerArgs);
            }
            if (this._performed) {
                this._performed(this, outerArgs);
            }
        };
        this.i.performed = delegateCombine(this.i.performed, this._performed_wrapped);
        ;
    }
}
