import * as React from 'react';
import { stringToColor, colorToString, fromSize, ensureEnum, brushToString, stringToBrush, ensureBool, initializePropertiesFromCss, NamePatcher, isValidProp, toSpinal, getModifiedProps, fromBrushCollection, toBrushCollection } from "igniteui-react-core";
import { FontInfo } from "igniteui-react-core";
import { ReactRenderer, PortalManager } from "igniteui-react-core";
import { delegateCombine, delegateRemove } from "igniteui-react-core";
import { ContentChildrenManager } from "igniteui-react-core";
import { CollectionAdapter } from "igniteui-react-core";
import { NotifyCollectionChangedAction } from "igniteui-react-core";
import { ToolPanel } from './ToolPanel';
import { BaseControlTheme_$type } from "igniteui-react-core";
import { ControlDisplayDensity_$type } from "igniteui-react-core";
import { ToolPanelOrientation_$type } from './ToolPanelOrientation';
import { ToolAction } from "./ToolAction";
import { IgrToolActionCollection } from "./igr-tool-action-collection";
import { IgrToolPanelContentRefreshedEventArgs } from "./igr-tool-panel-content-refreshed-event-args";
import { IgrToolCommandEventArgs } from './igr-tool-command-event-args';
import { NestedActionMode_$type } from './NestedActionMode';
export class IgrToolPanel extends React.Component {
    set height(value) {
        this._height = value;
        if (this._elRef) {
            this._elRef.style.height = value;
            this.i.notifySizeChanged();
        }
    }
    get height() {
        return this._height;
    }
    set width(value) {
        this._width = value;
        if (this._elRef) {
            this._elRef.style.height = value;
            this.i.notifySizeChanged();
        }
    }
    get width() {
        return this._width;
    }
    ensureStretch() {
        this.width = "100%";
        this.height = "100%";
    }
    _getMainRef(ref) {
        this._elRef = ref;
        this.verifyReferences();
    }
    _updateContentChildren() {
        this.contentActions.length = 0;
        let contentChildrenActual = this._contentChildrenManager.contentChildrenActual;
        for (let i = 0; i < contentChildrenActual.length; i++) {
            if ((ToolAction.$type).isAssignableFrom(contentChildrenActual[i].i.$type)) {
                this.contentActions.push(contentChildrenActual[i]);
            }
        }
        if (this._actionsAdapter !== null) {
            this._actionsAdapter.notifyContentChanged();
        }
    }
    /**
     * A collection of manually added style mappings for the treemap.
    */
    get actions() {
        if (this._actions === null) {
            let coll = new IgrToolActionCollection();
            let inner = coll._innerColl;
            inner.addListener((sender, e) => {
                switch (e.action) {
                    case NotifyCollectionChangedAction.Add:
                        this._actionsAdapter.insertManualItem(e.newStartingIndex, e.newItems.item(0));
                        break;
                    case NotifyCollectionChangedAction.Remove:
                        this._actionsAdapter.removeManualItemAt(e.oldStartingIndex);
                        break;
                    case NotifyCollectionChangedAction.Replace:
                        this._actionsAdapter.removeManualItemAt(e.oldStartingIndex);
                        this._actionsAdapter.insertManualItem(e.newStartingIndex, e.newItems.item(0));
                        break;
                    case NotifyCollectionChangedAction.Reset:
                        this._actionsAdapter.clearManualItems();
                        break;
                }
            });
            this._actions = coll;
        }
        return this._actions;
    }
    render() {
        let children = this._contentChildrenManager.getChildren(this.props.children);
        if (this._portalManager)
            this._portalManager.onRender(children);
        let div = React.createElement("div", {
            ref: this._getMainRef,
            className: "ig-tool-panel igr-tool-panel",
            children: children
        });
        return div;
    }
    verifyReferences() {
        if (this._elRef) {
            this.requestRender = this.requestRender.bind(this);
            this._portalManager = new PortalManager("toolPanelContent", this.requestRender);
            this._portalManager.disableContentPortal = true;
            this._reactRenderer = new ReactRenderer(this._elRef, document, true, {}, this._portalManager);
            if (document) {
                this._elRef.style.display = "block";
                this._elRef.style.width = "100%";
                this._elRef.style.height = "100%";
            }
            // render needs to fire again after the portalManager is created. 
            this.requestRender();
            this.i.provideContainer(this._reactRenderer);
            this.i.notifySizeChanged();
            this._reactRenderer.addSizeWatcher(() => {
                this.i.notifySizeChanged();
            });
        }
    }
    requestRender() {
        if (this._initialized)
            this.setState({});
    }
    constructor(props) {
        super(props);
        this._implementation = null;
        this._reactRenderer = null;
        this.contentActions = [];
        /**
        * The style mappings actually present in the treemap. Do not directly modify this array.
        * This array's contents can be modified by causing React to reproject the child content.
        * Or adding and removing ranges from the manual ranges collection on the ranges property.
        */
        this.actualActions = [];
        this._actions = null;
        this._actionsAdapter = null;
        this.__p = null;
        this._hasUserValues = new Set();
        this._stylingContainer = null;
        this._stylingParent = null;
        this._inStyling = false;
        this._contentRefreshed = null;
        this._contentRefreshed_wrapped = null;
        this._onCommand = null;
        this._onCommand_wrapped = null;
        if (this._styling) {
            NamePatcher.ensureStylablePatched(Object.getPrototypeOf(this));
        }
        this._implementation = this.createImplementation();
        this._implementation.externalObject = this;
        this._getMainRef = this._getMainRef.bind(this);
        this._contentChildrenManager = new ContentChildrenManager((ch) => ch.key || ch.props.name, (ch) => ch.key || ch.props.name, () => this._updateContentChildren());
        this._actionsAdapter = new CollectionAdapter(this.contentActions, this.i.actions, this.actualActions, (c) => c.i, (i) => {
            if (this._reactRenderer && this._elRef) {
                i._styling(this._elRef, this, this);
            }
        }, (i) => { });
        //var multiColumnComboBox = this.i;
        //multiColumnComboBox.notifySizeChanged();
        if (props) {
            this.initializeProperties();
        }
        this._initialized = true;
    }
    shouldComponentUpdate(nextProps, nextState) {
        const mod = getModifiedProps(this.props, nextProps);
        for (const p of Object.keys(mod)) {
            if (isValidProp(this, p)) {
                this[p] = mod[p];
            }
        }
        return true;
    }
    initializeProperties() {
        for (const p of Object.keys(this.props)) {
            if (isValidProp(this, p)) {
                this[p] = this.props[p];
            }
        }
    }
    // supports angular themes or custom properties set in CSS
    updateStyle() {
        this._styling(this._elRef, this);
    }
    componentWillUnmount() {
        this.i.destroy();
        if (this._reactRenderer)
            this._reactRenderer.destroy();
    }
    componentDidMount() {
        this._elRef.style.width = this._width ? this._width : "";
        this._elRef.style.height = this._height ? this._height : "";
        this.i.notifySizeChanged();
        this.initializeContent();
    }
    initializeContent() {
        this._styling(this._elRef, this);
        if (this.actualActions && this.actualActions.length > 0) {
            var currProperty = this.actualActions;
            for (var i = 0; i < currProperty.length; i++) {
                currProperty[i]._styling(this._elRef, this, this);
            }
        }
        this.updateStyle();
        this.i.notifySizeChanged();
    }
    createImplementation() {
        return new ToolPanel();
    }
    get i() {
        return this._implementation;
    }
    /**
 * Gets or sets if the tool actions should display their submenus on mouse hover.
*/
    get showOnHover() {
        return this.i.ay;
    }
    set showOnHover(v) {
        this.i.ay = ensureBool(v);
    }
    /**
     * Gets or sets if the tool action icon buttons should display their tooltips on mouse hover.
    */
    get showTooltipOnHover() {
        return this.i.az;
    }
    set showTooltipOnHover(v) {
        this.i.az = ensureBool(v);
    }
    /**
     * Gets or sets the amount of time to wait when clicking on the icon menu button while the dropdown is opening. If a click was
     * executed within this buffer window the dropdown will be left open. Otherwise it will close it. Setting a negative value will
     * always keep the dropdown open during a hover open and will require an extra click to close.
    */
    get dropdownClickBuffer() {
        return this.i.cp;
    }
    set dropdownClickBuffer(v) {
        this.i.cp = +v;
    }
    /**
     * Gets or sets the text style for the panel.
    */
    get cellTextStyle() {
        if (this.i.ak == null) {
            return null;
        }
        return this.i.ak.fontString;
    }
    set cellTextStyle(v) {
        let fi = new FontInfo();
        fi.fontString = v;
        this.i.ak = fi;
    }
    /**
     * Gets or sets the subtitle text style for the panel.
    */
    get subtitleTextStyle() {
        if (this.i.am == null) {
            return null;
        }
        return this.i.am.fontString;
    }
    set subtitleTextStyle(v) {
        let fi = new FontInfo();
        fi.fontString = v;
        this.i.am = fi;
    }
    /**
     * Gets or sets the base built in theme to use for the checkbox.
    */
    get baseTheme() {
        return this.i.x;
    }
    set baseTheme(v) {
        this.i.x = ensureEnum(BaseControlTheme_$type, v);
    }
    /**
     * Gets or sets the display density to use for the toolbar.
    */
    get density() {
        return this.i.ac;
    }
    set density(v) {
        this.i.ac = ensureEnum(ControlDisplayDensity_$type, v);
    }
    /**
     * Gets the resolved display density to use for the toolbar.
    */
    get actualDensity() {
        return this.i.ab;
    }
    set actualDensity(v) {
        this.i.ab = ensureEnum(ControlDisplayDensity_$type, v);
    }
    /**
     * Gets or sets the color to use for the background of the component.
    */
    get backgroundColor() {
        return brushToString(this.i.iu);
    }
    set backgroundColor(v) {
        this.i.iu = stringToBrush(v);
    }
    get actualBackgroundColor() {
        return brushToString(this.i.h7);
    }
    set actualBackgroundColor(v) {
        this.i.h7 = stringToBrush(v);
    }
    /**
     * Gets or sets the background color for submenus.
    */
    get submenuBackgroundColor() {
        return brushToString(this.i.ja);
    }
    set submenuBackgroundColor(v) {
        this.i.ja = stringToBrush(v);
    }
    get actualSubmenuBackgroundColor() {
        return brushToString(this.i.im);
    }
    set actualSubmenuBackgroundColor(v) {
        this.i.im = stringToBrush(v);
    }
    /**
     * Gets or sets the group header background color.
    */
    get groupHeaderBackgroundColor() {
        return brushToString(this.i.i0);
    }
    set groupHeaderBackgroundColor(v) {
        this.i.i0 = stringToBrush(v);
    }
    get actualGroupHeaderBackgroundColor() {
        return brushToString(this.i.ib);
    }
    set actualGroupHeaderBackgroundColor(v) {
        this.i.ib = stringToBrush(v);
    }
    /**
     * Gets or sets the separator background color.
    */
    get separatorBackgroundColor() {
        return brushToString(this.i.i9);
    }
    set separatorBackgroundColor(v) {
        this.i.i9 = stringToBrush(v);
    }
    get actualSeparatorBackgroundColor() {
        return brushToString(this.i.il);
    }
    set actualSeparatorBackgroundColor(v) {
        this.i.il = stringToBrush(v);
    }
    /**
     * Gets or sets the group header separator background color.
    */
    get groupHeaderSeparatorBackgroundColor() {
        return brushToString(this.i.i1);
    }
    set groupHeaderSeparatorBackgroundColor(v) {
        this.i.i1 = stringToBrush(v);
    }
    get actualGroupHeaderSeparatorBackgroundColor() {
        return brushToString(this.i.ic);
    }
    set actualGroupHeaderSeparatorBackgroundColor(v) {
        this.i.ic = stringToBrush(v);
    }
    /**
     * Gets or sets the hover background color.
    */
    get hoverBackgroundColor() {
        return brushToString(this.i.i5);
    }
    set hoverBackgroundColor(v) {
        this.i.i5 = stringToBrush(v);
    }
    get actualHoverBackgroundColor() {
        return brushToString(this.i.ih);
    }
    set actualHoverBackgroundColor(v) {
        this.i.ih = stringToBrush(v);
    }
    /**
     * Gets or sets the color to use for the text of the component.
    */
    get textColor() {
        return brushToString(this.i.jc);
    }
    set textColor(v) {
        this.i.jc = stringToBrush(v);
    }
    get actualTextColor() {
        return brushToString(this.i.ip);
    }
    set actualTextColor(v) {
        this.i.ip = stringToBrush(v);
    }
    get actualSubtitleTextColor() {
        return brushToString(this.i.io);
    }
    set actualSubtitleTextColor(v) {
        this.i.io = stringToBrush(v);
    }
    /**
     * Gets or sets the color to use for the subtitle of the component.
    */
    get subtitleTextColor() {
        return brushToString(this.i.jb);
    }
    set subtitleTextColor(v) {
        this.i.jb = stringToBrush(v);
    }
    /**
     * Gets or sets the color to use for the text in disabled tool actions.
    */
    get disabledTextColor() {
        return brushToString(this.i.ix);
    }
    set disabledTextColor(v) {
        this.i.ix = stringToBrush(v);
    }
    get actualDisabledTextColor() {
        return brushToString(this.i.ia);
    }
    set actualDisabledTextColor(v) {
        this.i.ia = stringToBrush(v);
    }
    /**
     * Gets or sets the text color for group header tool actions.
    */
    get groupHeaderTextColor() {
        return brushToString(this.i.i3);
    }
    set groupHeaderTextColor(v) {
        this.i.i3 = stringToBrush(v);
    }
    get actualGroupHeaderTextColor() {
        return brushToString(this.i.ie);
    }
    set actualGroupHeaderTextColor(v) {
        this.i.ie = stringToBrush(v);
    }
    /**
     * Gets or sets the text color for group header subtitle.
    */
    get groupHeaderSubtitleTextColor() {
        return brushToString(this.i.i2);
    }
    set groupHeaderSubtitleTextColor(v) {
        this.i.i2 = stringToBrush(v);
    }
    get actualGroupHeaderSubtitleTextColor() {
        return brushToString(this.i.id);
    }
    set actualGroupHeaderSubtitleTextColor(v) {
        this.i.id = stringToBrush(v);
    }
    /**
     * Gets or sets the tooltip text color.
    */
    get toolTipTextColor() {
        return brushToString(this.i.je);
    }
    set toolTipTextColor(v) {
        this.i.je = stringToBrush(v);
    }
    get actualToolTipTextColor() {
        return brushToString(this.i.ir);
    }
    set actualToolTipTextColor(v) {
        this.i.ir = stringToBrush(v);
    }
    /**
     * Gets or sets the tooltip background color.
    */
    get toolTipBackgroundColor() {
        return brushToString(this.i.jd);
    }
    set toolTipBackgroundColor(v) {
        this.i.jd = stringToBrush(v);
    }
    /**
     * Gets the actual tooltip background color.
    */
    get actualToolTipBackgroundColor() {
        return brushToString(this.i.iq);
    }
    set actualToolTipBackgroundColor(v) {
        this.i.iq = stringToBrush(v);
    }
    /**
     * Gets or sets the tooltip corner radius.
    */
    get toolTipCornerRadius() {
        return this.i.ci;
    }
    set toolTipCornerRadius(v) {
        this.i.ci = +v;
    }
    /**
     * Gets the actual tooltip corner radius.
    */
    get actualToolTipCornerRadius() {
        return this.i.b4;
    }
    set actualToolTipCornerRadius(v) {
        this.i.b4 = +v;
    }
    /**
     * Gets or sets the shadow elevation for the tooltip.
    */
    get toolTipElevation() {
        return this.i.cj;
    }
    set toolTipElevation(v) {
        this.i.cj = +v;
    }
    /**
     * Gets the actual shadow elevation used by the tooltip.
    */
    get actualToolTipElevation() {
        return this.i.b5;
    }
    set actualToolTipElevation(v) {
        this.i.b5 = +v;
    }
    /**
     * Gets or sets the fill color of icons in the ToolPanel.
    */
    get iconFill() {
        return brushToString(this.i.i6);
    }
    set iconFill(v) {
        this.i.i6 = stringToBrush(v);
    }
    /**
     * Gets the actual icon fill color.
    */
    get actualIconFill() {
        return brushToString(this.i.ii);
    }
    set actualIconFill(v) {
        this.i.ii = stringToBrush(v);
    }
    /**
     * Gets or sets the stroke color of icons in the ToolPanel.
    */
    get iconStroke() {
        return brushToString(this.i.i7);
    }
    set iconStroke(v) {
        this.i.i7 = stringToBrush(v);
    }
    /**
     * Gets the actual icon stroke color.
    */
    get actualIconStroke() {
        return brushToString(this.i.ij);
    }
    set actualIconStroke(v) {
        this.i.ij = stringToBrush(v);
    }
    /**
     * Gets or sets the left padding for separators in the vertical orientation.
    */
    get separatorVerticalPaddingLeft() {
        return this.i.cf;
    }
    set separatorVerticalPaddingLeft(v) {
        this.i.cf = +v;
    }
    get actualSeparatorVerticalPaddingLeft() {
        return this.i.b1;
    }
    set actualSeparatorVerticalPaddingLeft(v) {
        this.i.b1 = +v;
    }
    /**
     * Gets or sets the right padding for separators in the vertical orientation.
    */
    get separatorVerticalPaddingRight() {
        return this.i.cg;
    }
    set separatorVerticalPaddingRight(v) {
        this.i.cg = +v;
    }
    get actualSeparatorVerticalPaddingRight() {
        return this.i.b2;
    }
    set actualSeparatorVerticalPaddingRight(v) {
        this.i.b2 = +v;
    }
    /**
     * Gets or sets the top padding for separators in the vertical orientation.
    */
    get separatorVerticalPaddingTop() {
        return this.i.ch;
    }
    set separatorVerticalPaddingTop(v) {
        this.i.ch = +v;
    }
    get actualSeparatorVerticalPaddingTop() {
        return this.i.b3;
    }
    set actualSeparatorVerticalPaddingTop(v) {
        this.i.b3 = +v;
    }
    /**
     * Gets or sets the bottom padding for separators in the vertical orientation.
    */
    get separatorVerticalPaddingBottom() {
        return this.i.ce;
    }
    set separatorVerticalPaddingBottom(v) {
        this.i.ce = +v;
    }
    get actualSeparatorVerticalPaddingBottom() {
        return this.i.b0;
    }
    set actualSeparatorVerticalPaddingBottom(v) {
        this.i.b0 = +v;
    }
    /**
     * Gets or sets the left padding for separators in the horizontal orientation.
    */
    get separatorHorizontalPaddingLeft() {
        return this.i.cb;
    }
    set separatorHorizontalPaddingLeft(v) {
        this.i.cb = +v;
    }
    get actualSeparatorHorizontalPaddingLeft() {
        return this.i.bx;
    }
    set actualSeparatorHorizontalPaddingLeft(v) {
        this.i.bx = +v;
    }
    /**
     * Gets or sets the right padding for separators in the horizontal orientation.
    */
    get separatorHorizontalPaddingRight() {
        return this.i.cc;
    }
    set separatorHorizontalPaddingRight(v) {
        this.i.cc = +v;
    }
    get actualSeparatorHorizontalPaddingRight() {
        return this.i.by;
    }
    set actualSeparatorHorizontalPaddingRight(v) {
        this.i.by = +v;
    }
    /**
     * Gets or sets the top padding for separators in the horizontal orientation.
    */
    get separatorHorizontalPaddingTop() {
        return this.i.cd;
    }
    set separatorHorizontalPaddingTop(v) {
        this.i.cd = +v;
    }
    get actualSeparatorHorizontalPaddingTop() {
        return this.i.bz;
    }
    set actualSeparatorHorizontalPaddingTop(v) {
        this.i.bz = +v;
    }
    /**
     * Gets or sets the bottom padding for separators in the horizontal orientation.
    */
    get separatorHorizontalPaddingBottom() {
        return this.i.ca;
    }
    set separatorHorizontalPaddingBottom(v) {
        this.i.ca = +v;
    }
    get actualSeparatorHorizontalPaddingBottom() {
        return this.i.bw;
    }
    set actualSeparatorHorizontalPaddingBottom(v) {
        this.i.bw = +v;
    }
    /**
     * Gets or sets the color to use for the background of the component.
    */
    get orientation() {
        return this.i.u;
    }
    set orientation(v) {
        this.i.u = ensureEnum(ToolPanelOrientation_$type, v);
    }
    /**
     * Gets or sets the row height for the panel.
    */
    get rowHeight() {
        return this.i.ct;
    }
    set rowHeight(v) {
        this.i.ct = +v;
    }
    /**
     * Gets the actual row height used by the panel.
    */
    get actualRowHeight() {
        return this.i.bv;
    }
    get highlightWidth() {
        return this.i.b9;
    }
    set highlightWidth(v) {
        this.i.b9 = +v;
    }
    get actualHighlightWidth() {
        return this.i.bu;
    }
    set actualHighlightWidth(v) {
        this.i.bu = +v;
    }
    get highlightRadius() {
        return this.i.b8;
    }
    set highlightRadius(v) {
        this.i.b8 = +v;
    }
    get actualHighlightRadius() {
        return this.i.bt;
    }
    set actualHighlightRadius(v) {
        this.i.bt = +v;
    }
    get highlightColor() {
        return brushToString(this.i.i4);
    }
    set highlightColor(v) {
        this.i.i4 = stringToBrush(v);
    }
    get actualHighlightColor() {
        return brushToString(this.i.ig);
    }
    set actualHighlightColor(v) {
        this.i.ig = stringToBrush(v);
    }
    /**
     * Gets or sets the stroke color for all icon menu dropdown arrows.
    */
    get menuArrowStroke() {
        return brushToString(this.i.i8);
    }
    set menuArrowStroke(v) {
        this.i.i8 = stringToBrush(v);
    }
    /**
     * Gets the actual stroke color for all icon menu dropdown arrows.
    */
    get actualMenuArrowStroke() {
        return brushToString(this.i.ik);
    }
    set actualMenuArrowStroke(v) {
        this.i.ik = stringToBrush(v);
    }
    /**
     * Gets or sets the background color of the radio when in the unchecked state.
    */
    get uncheckedBackgroundColor() {
        return brushToString(this.i.jf);
    }
    set uncheckedBackgroundColor(v) {
        this.i.jf = stringToBrush(v);
    }
    /**
     * Gets the actual background color of the radio when in the unchecked state.
    */
    get actualUncheckedBackgroundColor() {
        return brushToString(this.i.is);
    }
    set actualUncheckedBackgroundColor(v) {
        this.i.is = stringToBrush(v);
    }
    /**
     * Gets or sets the background color of the radio when in the checked state.
    */
    get checkedBackgroundColor() {
        return brushToString(this.i.iv);
    }
    set checkedBackgroundColor(v) {
        this.i.iv = stringToBrush(v);
    }
    /**
     * Gets the actual background color of the radio when in the checked state.
    */
    get actualCheckedBackgroundColor() {
        return brushToString(this.i.h8);
    }
    set actualCheckedBackgroundColor(v) {
        this.i.h8 = stringToBrush(v);
    }
    /**
     * Gets or sets the border color of the radio when in the unchecked state.
    */
    get uncheckedBorderColor() {
        return brushToString(this.i.jg);
    }
    set uncheckedBorderColor(v) {
        this.i.jg = stringToBrush(v);
    }
    /**
     * Gets the actual border color of the radio when in the unchecked state.
    */
    get actualUncheckedBorderColor() {
        return brushToString(this.i.it);
    }
    set actualUncheckedBorderColor(v) {
        this.i.it = stringToBrush(v);
    }
    /**
     * Gets or sets the border color of the radio when in the checked state.
    */
    get checkedBorderColor() {
        return brushToString(this.i.iw);
    }
    set checkedBorderColor(v) {
        this.i.iw = stringToBrush(v);
    }
    /**
     * Gets the actual border color of the radio when in the checked state.
    */
    get actualCheckedBorderColor() {
        return brushToString(this.i.h9);
    }
    set actualCheckedBorderColor(v) {
        this.i.h9 = stringToBrush(v);
    }
    /**
     * Gets or sets the dropdown delay time in milliseconds.
    */
    get dropdownDelay() {
        return this.i.cq;
    }
    set dropdownDelay(v) {
        this.i.cq = +v;
    }
    /**
     * Gets the actual dropdown delay.
    */
    get actualDropdownDelay() {
        return this.i.co;
    }
    set actualDropdownDelay(v) {
        this.i.co = +v;
    }
    /**
     * Gets or sets the mode used when displaying nested actions.
    */
    get nestedActionMode() {
        return this.i.g;
    }
    set nestedActionMode(v) {
        this.i.g = ensureEnum(NestedActionMode_$type, v);
    }
    get isOpen() {
        return this.i.ax;
    }
    findByName(name) {
        if (this.findEphemera) {
            if (name && name.indexOf("@@e:") == 0) {
                return this.findEphemera(name);
            }
        }
        if (this.actions != null && this.actions.findByName && this.actions.findByName(name)) {
            return this.actions.findByName(name);
        }
        return null;
    }
    get hasUserValues() {
        return this._hasUserValues;
    }
    __m(propertyName) {
        if (!this._inStyling) {
            this._hasUserValues.add(propertyName);
        }
    }
    _styling(container, component, parent) {
        if (this._inStyling) {
            return;
        }
        this._inStyling = true;
        this._stylingContainer = container;
        this._stylingParent = component;
        let genericPrefix = "";
        let typeName = this.i.$type.name;
        if (typeName.indexOf("Xam") === 0) {
            typeName = typeName.substring(3);
        }
        genericPrefix = toSpinal("ToolPanel");
        let additionalPrefixes = [];
        let prefix = toSpinal(typeName);
        additionalPrefixes.push(prefix + "-");
        let b = this.i.$type.baseType;
        while (b && b.name != "Object" &&
            b.name != "Base" &&
            b.name != "Control" &&
            b.Name != "DependencyObject" &&
            b.Name != "FrameworkElement") {
            typeName = b.name;
            if (typeName.indexOf("Xam") === 0) {
                typeName = typeName.substring(3);
            }
            let basePrefix = toSpinal(typeName);
            additionalPrefixes.push(basePrefix + "-");
            b = b.baseType;
        }
        if (parent) {
            let parentTypeName = parent.i.$type.name;
            if (parentTypeName.indexOf("Xam") === 0) {
                parentTypeName = parentTypeName.substring(3);
            }
            let parentPrefix = toSpinal(parentTypeName);
            additionalPrefixes.push(parentPrefix + "-" + genericPrefix + "-");
            additionalPrefixes.push(parentPrefix + "-" + prefix + "-");
        }
        initializePropertiesFromCss(container, this, genericPrefix + "-", this.hasUserValues, false, additionalPrefixes);
        if (this._otherStyling) {
            this._otherStyling(container, component, parent);
        }
        this._inStyling = false;
    }
    refresh() {
        this.i.f9();
    }
    getDesiredSize() {
        let iv = this.i.jj();
        return fromSize(iv);
    }
    flushRefresh() {
        this.i.fr();
    }
    /**
     * Forces any open sub menus to close.
    
    */
    closeSubmenus() {
        this.i.fm();
    }
    exportVisualData() {
        let iv = this.i.cu();
        return (iv);
    }
    exportSerializedVisualData() {
        let iv = this.i.ee();
        return (iv);
    }
    setBrushContextItem(key, value) {
        this.i.gi(key, stringToBrush(value));
    }
    getBrushContextItem(key) {
        let iv = this.i.iy(key);
        return brushToString(iv);
    }
    setBrushCollectionContextItem(key, value) {
        this.i.gg(key, toBrushCollection(value));
    }
    getBrushCollectionContextItem(key) {
        let iv = this.i.a(key);
        return fromBrushCollection(iv);
    }
    setDoubleContextItem(key, value) {
        this.i.gq(key, value);
    }
    getDoubleContextItem(key) {
        let iv = this.i.b6(key);
        return (iv);
    }
    setIntContextItem(key, value) {
        this.i.gs(key, value);
    }
    getIntContextItem(key) {
        let iv = this.i.cr(key);
        return (iv);
    }
    setStringContextItem(key, value) {
        this.i.gu(key, value);
    }
    getStringContextItem(key) {
        let iv = this.i.ef(key);
        return (iv);
    }
    setBoolContextItem(key, value) {
        this.i.ge(key, value);
    }
    getBoolContextItem(key) {
        let iv = this.i.au(key);
        return (iv);
    }
    setColorContextItem(key, value) {
        this.i.gk(key, stringToColor(value));
    }
    getColorContextItem(key) {
        let iv = this.i.jh(key);
        return colorToString(iv);
    }
    setDataContextItem(key, value) {
        this.i.go(key, value);
    }
    getDataContextItem(key) {
        let iv = this.i.cx(key);
        return (iv);
    }
    get contentRefreshed() {
        return this._contentRefreshed;
    }
    set contentRefreshed(ev) {
        if (this._contentRefreshed_wrapped !== null) {
            this.i.contentRefreshed = delegateRemove(this.i.contentRefreshed, this._contentRefreshed_wrapped);
            this._contentRefreshed_wrapped = null;
            this._contentRefreshed = null;
        }
        this._contentRefreshed = ev;
        this._contentRefreshed_wrapped = (o, e) => {
            let outerArgs = new IgrToolPanelContentRefreshedEventArgs();
            outerArgs._provideImplementation(e);
            if (this.beforeContentRefreshed) {
                this.beforeContentRefreshed(this, outerArgs);
            }
            if (this._contentRefreshed) {
                this._contentRefreshed(this, outerArgs);
            }
        };
        this.i.contentRefreshed = delegateCombine(this.i.contentRefreshed, this._contentRefreshed_wrapped);
        ;
    }
    get onCommand() {
        return this._onCommand;
    }
    set onCommand(ev) {
        if (this._onCommand_wrapped !== null) {
            this.i.onCommand = delegateRemove(this.i.onCommand, this._onCommand_wrapped);
            this._onCommand_wrapped = null;
            this._onCommand = null;
        }
        this._onCommand = ev;
        this._onCommand_wrapped = (o, e) => {
            let outerArgs = new IgrToolCommandEventArgs();
            outerArgs._provideImplementation(e);
            if (this.beforeOnCommand) {
                this.beforeOnCommand(this, outerArgs);
            }
            if (this._onCommand) {
                this._onCommand(this, outerArgs);
            }
        };
        this.i.onCommand = delegateCombine(this.i.onCommand, this._onCommand_wrapped);
        ;
    }
}
