import * as React from 'react';
import { stringToColor, colorToString, fromSize, ensureEnum, brushToString, stringToBrush, ensureBool, initializePropertiesFromCss, NamePatcher, isValidProp, toSpinal, getModifiedProps, fromBrushCollection, toBrushCollection } from "igniteui-react-core";
import { FontInfo } from "igniteui-react-core";
import { ReactRenderer, PortalManager } from "igniteui-react-core";
import { delegateCombine, delegateRemove } from "igniteui-react-core";
import { ContentChildrenManager } from "igniteui-react-core";
import { CollectionAdapter } from "igniteui-react-core";
import { NotifyCollectionChangedAction } from "igniteui-react-core";
import { Toolbar } from './Toolbar';
import { BaseControlTheme_$type } from "igniteui-react-core";
import { ControlDisplayDensity_$type } from "igniteui-react-core";
import { SyncableObservableCollection$1 } from "igniteui-react-core";
import { ToolbarOrientation_$type } from './ToolbarOrientation';
import { ToolActionCollection as ToolActionCollection_internal } from "./ToolActionCollection";
import { ToolAction } from "./ToolAction";
import { IgrToolActionCollection } from "./igr-tool-action-collection";
import { IgrToolCommandEventArgs } from './igr-tool-command-event-args';
import { IgrToolbarSubMenuOpeningEventArgs } from './igr-toolbar-sub-menu-opening-event-args';
import { IgrToolbarSubMenuClosingEventArgs } from './igr-toolbar-sub-menu-closing-event-args';
export class IgrToolbar extends React.Component {
    set height(value) {
        this._height = value;
        this.i.userSetHeight = value ? true : false;
        if (this._elRef) {
            this._elRef.style.height = value;
            this.i.notifySizeChanged();
        }
    }
    get height() {
        return this._height;
    }
    set width(value) {
        this._width = value;
        this.i.userSetWidth = value ? true : false;
        if (this._elRef) {
            this._elRef.style.height = value;
            this.i.notifySizeChanged();
        }
    }
    get width() {
        return this._width;
    }
    _getMainRef(ref) {
        this._elRef = ref;
        this.verifyReferences();
    }
    _updateContentChildren() {
        this.contentActions.length = 0;
        let contentChildrenActual = this._contentChildrenManager.contentChildrenActual;
        for (let i = 0; i < contentChildrenActual.length; i++) {
            if ((ToolAction.$type).isAssignableFrom(contentChildrenActual[i].i.$type)) {
                this.contentActions.push(contentChildrenActual[i]);
            }
        }
        if (this._actionsAdapter !== null) {
            this._actionsAdapter.notifyContentChanged();
        }
    }
    /**
     * A collection of manually added style mappings for the treemap.
    */
    get actions() {
        if (this._actions === null) {
            let coll = new IgrToolActionCollection();
            let inner = coll._innerColl;
            inner.addListener((sender, e) => {
                switch (e.action) {
                    case NotifyCollectionChangedAction.Add:
                        this._actionsAdapter.insertManualItem(e.newStartingIndex, e.newItems.item(0));
                        break;
                    case NotifyCollectionChangedAction.Remove:
                        this._actionsAdapter.removeManualItemAt(e.oldStartingIndex);
                        break;
                    case NotifyCollectionChangedAction.Replace:
                        this._actionsAdapter.removeManualItemAt(e.oldStartingIndex);
                        this._actionsAdapter.insertManualItem(e.newStartingIndex, e.newItems.item(0));
                        break;
                    case NotifyCollectionChangedAction.Reset:
                        this._actionsAdapter.clearManualItems();
                        break;
                }
            });
            this._actions = coll;
        }
        return this._actions;
    }
    render() {
        let children = this._contentChildrenManager.getChildren(this.props.children);
        if (this._portalManager)
            this._portalManager.onRender(children);
        let div = React.createElement("div", {
            ref: this._getMainRef,
            className: "ig-toolbar igr-toolbar",
            children: children
        });
        return div;
    }
    verifyReferences() {
        if (this._elRef) {
            this.requestRender = this.requestRender.bind(this);
            this._portalManager = new PortalManager("toolbarContent", this.requestRender);
            this._reactRenderer = new ReactRenderer(this._elRef, document, true, {}, this._portalManager);
            if (document) {
                this._elRef.style.display = "block";
                this._elRef.style.width = "100%";
                this._elRef.style.height = "100%";
            }
            // render needs to fire again after the portalManager is created. 
            this.requestRender();
            this.i.provideContainer(this._reactRenderer);
            this.i.notifySizeChanged();
            this._reactRenderer.addSizeWatcher(() => {
                this.i.notifySizeChanged();
            });
        }
    }
    requestRender() {
        if (this._initialized)
            this.setState({});
    }
    constructor(props) {
        super(props);
        this._implementation = null;
        this._reactRenderer = null;
        this.contentActions = [];
        /**
        * The style mappings actually present in the treemap. Do not directly modify this array.
        * This array's contents can be modified by causing React to reproject the child content.
        * Or adding and removing ranges from the manual ranges collection on the ranges property.
        */
        this.combinedActions = [];
        this._actions = null;
        this._actionsAdapter = null;
        this._autoGeneratedActions = null;
        this._actualActions = null;
        this.__p = null;
        this._hasUserValues = new Set();
        this._stylingContainer = null;
        this._stylingParent = null;
        this._inStyling = false;
        this._onCommand = null;
        this._onCommand_wrapped = null;
        this._subMenuOpening = null;
        this._subMenuOpening_wrapped = null;
        this._subMenuClosing = null;
        this._subMenuClosing_wrapped = null;
        if (this._styling) {
            NamePatcher.ensureStylablePatched(Object.getPrototypeOf(this));
        }
        this._implementation = this.createImplementation();
        this._implementation.externalObject = this;
        this._getMainRef = this._getMainRef.bind(this);
        this._contentChildrenManager = new ContentChildrenManager((ch) => ch.key || ch.props.name, (ch) => ch.key || ch.props.name, () => this._updateContentChildren());
        this._actionsAdapter = new CollectionAdapter(this.contentActions, this.i.actions, this.combinedActions, (c) => c.i, (i) => {
            if (this._reactRenderer && this._elRef) {
                i._styling(this._elRef, this, this);
            }
        }, (i) => { });
        //var multiColumnComboBox = this.i;
        //multiColumnComboBox.notifySizeChanged();
        if (props) {
            this.initializeProperties();
        }
        this._initialized = true;
    }
    shouldComponentUpdate(nextProps, nextState) {
        const mod = getModifiedProps(this.props, nextProps);
        for (const p of Object.keys(mod)) {
            if (isValidProp(this, p)) {
                this[p] = mod[p];
            }
        }
        return true;
    }
    initializeProperties() {
        for (const p of Object.keys(this.props)) {
            if (isValidProp(this, p)) {
                this[p] = this.props[p];
            }
        }
    }
    // supports angular themes or custom properties set in CSS
    updateStyle() {
        this._styling(this._elRef, this);
    }
    componentWillUnmount() {
        this.i.destroy();
        if (this._reactRenderer)
            this._reactRenderer.destroy();
    }
    componentDidMount() {
        this._elRef.style.width = this._width ? this._width : "";
        this._elRef.style.height = this._height ? this._height : "";
        this.i.notifySizeChanged();
        this.initializeContent();
    }
    initializeContent() {
        this._styling(this._elRef, this);
        if (this.combinedActions && this.combinedActions.length > 0) {
            var currProperty = this.combinedActions;
            for (var i = 0; i < currProperty.length; i++) {
                currProperty[i]._styling(this._elRef, this, this);
            }
        }
        this.updateStyle();
        this.i.notifySizeChanged();
    }
    createImplementation() {
        return new Toolbar();
    }
    get i() {
        return this._implementation;
    }
    /**
     * Gets or sets a target component which the toolbar can request actions from.
     */
    get target() {
        var ret = this.i.target;
        if (!ret) {
            return ret;
        }
        if (ret.externalObject != null) {
            ret = ret.externalObject;
        }
        return ret;
    }
    set target(v) {
        if (v && v._implementation) {
            v = v._implementation;
        }
        this.i.target = v;
    }
    /**
 * Gets or sets the base built in theme to use for the checkbox.
*/
    get baseTheme() {
        return this.i.v;
    }
    set baseTheme(v) {
        this.i.v = ensureEnum(BaseControlTheme_$type, v);
    }
    /**
     * Gets or sets the display density to use for the toolbar.
    */
    get density() {
        return this.i.x;
    }
    set density(v) {
        this.i.x = ensureEnum(ControlDisplayDensity_$type, v);
    }
    /**
     * Gets or sets the data or data source instance to which to bind the grid.
    */
    get autoGeneratedActions() {
        if (this._autoGeneratedActions === null) {
            let coll = new IgrToolActionCollection();
            let innerColl = this.i.autoGeneratedActions;
            if (!innerColl) {
                innerColl = new ToolActionCollection_internal();
            }
            this._autoGeneratedActions = coll._fromInner(innerColl);
            this.i.autoGeneratedActions = innerColl;
        }
        return this._autoGeneratedActions;
    }
    set autoGeneratedActions(v) {
        if (this._autoGeneratedActions !== null) {
            this._autoGeneratedActions._setSyncTarget(null);
            this._autoGeneratedActions = null;
        }
        let coll = new IgrToolActionCollection();
        this._autoGeneratedActions = coll._fromOuter(v);
        let syncColl = new SyncableObservableCollection$1(ToolAction.$type);
        let innerColl = this.i.autoGeneratedActions;
        if (!innerColl) {
            innerColl = new ToolActionCollection_internal();
        }
        syncColl._inner = innerColl;
        syncColl.clear();
        this._autoGeneratedActions._setSyncTarget(syncColl);
        this.i.autoGeneratedActions = innerColl;
    }
    /**
     * Gets or sets the data or data source instance to which to bind the grid.
    */
    get actualActions() {
        if (this._actualActions === null) {
            let coll = new IgrToolActionCollection();
            let innerColl = this.i.actualActions;
            if (!innerColl) {
                innerColl = new ToolActionCollection_internal();
            }
            this._actualActions = coll._fromInner(innerColl);
            this.i.actualActions = innerColl;
        }
        return this._actualActions;
    }
    set actualActions(v) {
        if (this._actualActions !== null) {
            this._actualActions._setSyncTarget(null);
            this._actualActions = null;
        }
        let coll = new IgrToolActionCollection();
        this._actualActions = coll._fromOuter(v);
        let syncColl = new SyncableObservableCollection$1(ToolAction.$type);
        let innerColl = this.i.actualActions;
        if (!innerColl) {
            innerColl = new ToolActionCollection_internal();
        }
        syncColl._inner = innerColl;
        syncColl.clear();
        this._actualActions._setSyncTarget(syncColl);
        this.i.actualActions = innerColl;
    }
    /**
     * Gets or sets the orientation of the toolbar.
    */
    get orientation() {
        return this.i.m;
    }
    set orientation(v) {
        this.i.m = ensureEnum(ToolbarOrientation_$type, v);
    }
    /**
     * Gets or sets the color to use for the background of the component.
    */
    get backgroundColor() {
        return brushToString(this.i.fe);
    }
    set backgroundColor(v) {
        this.i.fe = stringToBrush(v);
    }
    /**
     * Gets or sets the background color for submenus.
    */
    get submenuBackgroundColor() {
        return brushToString(this.i.fs);
    }
    set submenuBackgroundColor(v) {
        this.i.fs = stringToBrush(v);
    }
    /**
     * Gets or sets the group header background color.
    */
    get groupHeaderBackgroundColor() {
        return brushToString(this.i.fj);
    }
    set groupHeaderBackgroundColor(v) {
        this.i.fj = stringToBrush(v);
    }
    /**
     * Gets or sets the separator background color.
    */
    get separatorBackgroundColor() {
        return brushToString(this.i.fr);
    }
    set separatorBackgroundColor(v) {
        this.i.fr = stringToBrush(v);
    }
    /**
     * Gets or sets the hover background color.
    */
    get hoverBackgroundColor() {
        return brushToString(this.i.fn);
    }
    set hoverBackgroundColor(v) {
        this.i.fn = stringToBrush(v);
    }
    /**
     * Gets or sets the color to use for the text of the component.
    */
    get textColor() {
        return brushToString(this.i.fu);
    }
    set textColor(v) {
        this.i.fu = stringToBrush(v);
    }
    /**
     * Gets or sets the color to use for the text in disabled tool actions.
    */
    get disabledTextColor() {
        return brushToString(this.i.fh);
    }
    set disabledTextColor(v) {
        this.i.fh = stringToBrush(v);
    }
    /**
     * Gets or sets the text color for group header tool actions.
    */
    get groupHeaderTextColor() {
        return brushToString(this.i.fl);
    }
    set groupHeaderTextColor(v) {
        this.i.fl = stringToBrush(v);
    }
    /**
     * Gets or sets the tooltip text color.
    */
    get toolTipTextColor() {
        return brushToString(this.i.fw);
    }
    set toolTipTextColor(v) {
        this.i.fw = stringToBrush(v);
    }
    /**
     * Gets or sets the tooltip background color.
    */
    get toolTipBackgroundColor() {
        return brushToString(this.i.fv);
    }
    set toolTipBackgroundColor(v) {
        this.i.fv = stringToBrush(v);
    }
    /**
     * Gets or sets the tooltip corner radius.
    */
    get toolTipCornerRadius() {
        return this.i.bh;
    }
    set toolTipCornerRadius(v) {
        this.i.bh = +v;
    }
    /**
     * Gets or sets the shadow elevation for the tooltip.
    */
    get toolTipElevation() {
        return this.i.bi;
    }
    set toolTipElevation(v) {
        this.i.bi = +v;
    }
    /**
     * Gets or sets the fill color of icons in the ToolPanel.
    */
    get iconFill() {
        return brushToString(this.i.fo);
    }
    set iconFill(v) {
        this.i.fo = stringToBrush(v);
    }
    /**
     * Gets or sets the stroke color of icons in the ToolPanel.
    */
    get iconStroke() {
        return brushToString(this.i.fp);
    }
    set iconStroke(v) {
        this.i.fp = stringToBrush(v);
    }
    /**
     * Gets or sets the text style.
    */
    get cellTextStyle() {
        if (this.i.ab == null) {
            return null;
        }
        return this.i.ab.fontString;
    }
    set cellTextStyle(v) {
        let fi = new FontInfo();
        fi.fontString = v;
        this.i.ab = fi;
    }
    /**
     * Gets or sets the text style for subtitles.
    */
    get subtitleTextStyle() {
        if (this.i.ad == null) {
            return null;
        }
        return this.i.ad.fontString;
    }
    set subtitleTextStyle(v) {
        let fi = new FontInfo();
        fi.fontString = v;
        this.i.ad = fi;
    }
    /**
     * Gets or sets the left padding for separators in the vertical orientation.
    */
    get separatorVerticalPaddingLeft() {
        return this.i.be;
    }
    set separatorVerticalPaddingLeft(v) {
        this.i.be = +v;
    }
    /**
     * Gets or sets the right padding for separators in the vertical orientation.
    */
    get separatorVerticalPaddingRight() {
        return this.i.bf;
    }
    set separatorVerticalPaddingRight(v) {
        this.i.bf = +v;
    }
    /**
     * Gets or sets the top padding for separators in the vertical orientation.
    */
    get separatorVerticalPaddingTop() {
        return this.i.bg;
    }
    set separatorVerticalPaddingTop(v) {
        this.i.bg = +v;
    }
    /**
     * Gets or sets the bottom padding for separators in the vertical orientation.
    */
    get separatorVerticalPaddingBottom() {
        return this.i.bd;
    }
    set separatorVerticalPaddingBottom(v) {
        this.i.bd = +v;
    }
    /**
     * Gets or sets the left padding for separators in the horizontal orientation.
    */
    get separatorHorizontalPaddingLeft() {
        return this.i.ba;
    }
    set separatorHorizontalPaddingLeft(v) {
        this.i.ba = +v;
    }
    /**
     * Gets or sets the right padding for separators in the horizontal orientation.
    */
    get separatorHorizontalPaddingRight() {
        return this.i.bb;
    }
    set separatorHorizontalPaddingRight(v) {
        this.i.bb = +v;
    }
    /**
     * Gets or sets the top padding for separators in the horizontal orientation.
    */
    get separatorHorizontalPaddingTop() {
        return this.i.bc;
    }
    set separatorHorizontalPaddingTop(v) {
        this.i.bc = +v;
    }
    /**
     * Gets or sets the bottom padding for separators in the horizontal orientation.
    */
    get separatorHorizontalPaddingBottom() {
        return this.i.a9;
    }
    set separatorHorizontalPaddingBottom(v) {
        this.i.a9 = +v;
    }
    /**
     * Gets or sets if the tool actions should display their submenus on mouse hover.
    */
    get showOnHover() {
        return this.i.ap;
    }
    set showOnHover(v) {
        this.i.ap = ensureBool(v);
    }
    /**
     * Gets or sets if the tool action icon buttons should display their tooltips on mouse hover.
    */
    get showTooltipOnHover() {
        return this.i.aq;
    }
    set showTooltipOnHover(v) {
        this.i.aq = ensureBool(v);
    }
    /**
     * Gets or sets the amount of time to wait when clicking on the icon menu button while the dropdown is opening. If a click was
     * executed within this buffer window the dropdown will be left open. Otherwise it will close it. Setting a negative value will
     * always keep the dropdown open during a hover open and will require an extra click to close.
    */
    get dropdownClickBuffer() {
        return this.i.bm;
    }
    set dropdownClickBuffer(v) {
        this.i.bm = +v;
    }
    get highlightWidth() {
        return this.i.a8;
    }
    set highlightWidth(v) {
        this.i.a8 = +v;
    }
    get highlightRadius() {
        return this.i.a7;
    }
    set highlightRadius(v) {
        this.i.a7 = +v;
    }
    get highlightColor() {
        return brushToString(this.i.fm);
    }
    set highlightColor(v) {
        this.i.fm = stringToBrush(v);
    }
    /**
     * Gets or sets the stroke color for all icon menu dropdown arrows.
    */
    get menuArrowStroke() {
        return brushToString(this.i.fq);
    }
    set menuArrowStroke(v) {
        this.i.fq = stringToBrush(v);
    }
    /**
     * Gets or sets the background color of the radio when in the unchecked state.
    */
    get uncheckedBackgroundColor() {
        return brushToString(this.i.fx);
    }
    set uncheckedBackgroundColor(v) {
        this.i.fx = stringToBrush(v);
    }
    /**
     * Gets or sets the background color of the radio when in the checked state.
    */
    get checkedBackgroundColor() {
        return brushToString(this.i.ff);
    }
    set checkedBackgroundColor(v) {
        this.i.ff = stringToBrush(v);
    }
    /**
     * Gets or sets the border color of the radio when in the unchecked state.
    */
    get uncheckedBorderColor() {
        return brushToString(this.i.fy);
    }
    set uncheckedBorderColor(v) {
        this.i.fy = stringToBrush(v);
    }
    /**
     * Gets or sets the border color of the radio when in the checked state.
    */
    get checkedBorderColor() {
        return brushToString(this.i.fg);
    }
    set checkedBorderColor(v) {
        this.i.fg = stringToBrush(v);
    }
    /**
     * Gets or sets the row height that should be used.
    */
    get rowHeight() {
        return this.i.bq;
    }
    set rowHeight(v) {
        this.i.bq = +v;
    }
    /**
     * Gets or sets the text color for subtitles.
    */
    get subtitleTextColor() {
        return brushToString(this.i.ft);
    }
    set subtitleTextColor(v) {
        this.i.ft = stringToBrush(v);
    }
    /**
     * Gets or sets the text color for group header subtitles.
    */
    get groupHeaderSubtitleTextColor() {
        return brushToString(this.i.fk);
    }
    set groupHeaderSubtitleTextColor(v) {
        this.i.fk = stringToBrush(v);
    }
    /**
     * Gets or sets the dropdown delay time in milliseconds.
    */
    get dropdownDelay() {
        return this.i.bn;
    }
    set dropdownDelay(v) {
        this.i.bn = +v;
    }
    get isOpen() {
        return this.i.ao;
    }
    findByName(name) {
        if (this.findEphemera) {
            if (name && name.indexOf("@@e:") == 0) {
                return this.findEphemera(name);
            }
        }
        if (this.autoGeneratedActions != null && this.autoGeneratedActions.findByName && this.autoGeneratedActions.findByName(name)) {
            return this.autoGeneratedActions.findByName(name);
        }
        if (this.actions != null && this.actions.findByName && this.actions.findByName(name)) {
            return this.actions.findByName(name);
        }
        if (this.actualActions != null && this.actualActions.findByName && this.actualActions.findByName(name)) {
            return this.actualActions.findByName(name);
        }
        return null;
    }
    get hasUserValues() {
        return this._hasUserValues;
    }
    __m(propertyName) {
        if (!this._inStyling) {
            this._hasUserValues.add(propertyName);
        }
    }
    _styling(container, component, parent) {
        if (this._inStyling) {
            return;
        }
        this._inStyling = true;
        this._stylingContainer = container;
        this._stylingParent = component;
        let genericPrefix = "";
        let typeName = this.i.$type.name;
        if (typeName.indexOf("Xam") === 0) {
            typeName = typeName.substring(3);
        }
        genericPrefix = toSpinal("Toolbar");
        let additionalPrefixes = [];
        let prefix = toSpinal(typeName);
        additionalPrefixes.push(prefix + "-");
        let b = this.i.$type.baseType;
        while (b && b.name != "Object" &&
            b.name != "Base" &&
            b.name != "Control" &&
            b.Name != "DependencyObject" &&
            b.Name != "FrameworkElement") {
            typeName = b.name;
            if (typeName.indexOf("Xam") === 0) {
                typeName = typeName.substring(3);
            }
            let basePrefix = toSpinal(typeName);
            additionalPrefixes.push(basePrefix + "-");
            b = b.baseType;
        }
        if (parent) {
            let parentTypeName = parent.i.$type.name;
            if (parentTypeName.indexOf("Xam") === 0) {
                parentTypeName = parentTypeName.substring(3);
            }
            let parentPrefix = toSpinal(parentTypeName);
            additionalPrefixes.push(parentPrefix + "-" + genericPrefix + "-");
            additionalPrefixes.push(parentPrefix + "-" + prefix + "-");
        }
        initializePropertiesFromCss(container, this, genericPrefix + "-", this.hasUserValues, false, additionalPrefixes);
        if (this._otherStyling) {
            this._otherStyling(container, component, parent);
        }
        this._inStyling = false;
    }
    flushRefresh() {
        this.i.dr();
    }
    /**
     * Get the desired size the toolbar wants to be.
    
    */
    getDesiredSize() {
        let iv = this.i.f0();
        return fromSize(iv);
    }
    /**
     * Forces any open sub menus to close.
    
    */
    closeSubmenus() {
        this.i.dn();
    }
    onCommandStateChanged(id, type, value) {
        this.i.onCommandStateChanged(id, (type == null ? null : type), value);
    }
    registerIconFromText(iconCollection, iconName, iconText) {
        this.i.d8(iconCollection, iconName, iconText);
    }
    getIconFromCache(iconCollection, iconName) {
        let iv = this.i.ci(iconCollection, iconName);
        return (iv);
    }
    registerMultiPathSVG(iconCollection, iconName, multiPathSVG) {
        this.i.ea(iconCollection, iconName, multiPathSVG);
    }
    getMultiPathSVGFromCache(iconCollection, iconName) {
        let iv = this.i.a(iconCollection, iconName);
        return (iv);
    }
    registerIconFromDataURL(iconCollection, iconName, dataURL) {
        this.i.d7(iconCollection, iconName, dataURL);
    }
    getDataURLFromCache(iconCollection, iconName) {
        let iv = this.i.ch(iconCollection, iconName);
        return (iv);
    }
    registerIconSource(iconCollection, iconName, source) {
        this.i.d9(iconCollection, iconName, source);
    }
    getIconSource(iconCollection, iconName) {
        let iv = this.i.b0(iconCollection, iconName);
        return (iv);
    }
    setBrushContextItem(key, value) {
        this.i.ed(key, stringToBrush(value));
    }
    getBrushContextItem(key) {
        let iv = this.i.fi(key);
        return brushToString(iv);
    }
    setBrushCollectionContextItem(key, value) {
        this.i.ec(key, toBrushCollection(value));
    }
    getBrushCollectionContextItem(key) {
        let iv = this.i.b(key);
        return fromBrushCollection(iv);
    }
    setDoubleContextItem(key, value) {
        this.i.eh(key, value);
    }
    getDoubleContextItem(key) {
        let iv = this.i.a5(key);
        return (iv);
    }
    setIntContextItem(key, value) {
        this.i.eo(key, value);
    }
    getExternalDoubleContextItem(key) {
        let iv = this.i.a6(key);
        return (iv);
    }
    setExternalDoubleContextItem(key, value) {
        this.i.em(key, value);
    }
    getIntContextItem(key) {
        let iv = this.i.bp(key);
        return (iv);
    }
    setExternalIntContextItem(key, value) {
        this.i.en(key, value);
    }
    getExternalIntContextItem(key) {
        let iv = this.i.bo(key);
        return (iv);
    }
    setStringContextItem(key, value) {
        this.i.ep(key, value);
    }
    getStringContextItem(key) {
        let iv = this.i.cj(key);
        return (iv);
    }
    setBoolContextItem(key, value) {
        this.i.eb(key, value);
    }
    getBoolContextItem(key) {
        let iv = this.i.an(key);
        return (iv);
    }
    setColorContextItem(key, value) {
        this.i.ee(key, stringToColor(value));
    }
    getColorContextItem(key) {
        let iv = this.i.fz(key);
        return colorToString(iv);
    }
    setDataContextItem(key, value) {
        this.i.eg(key, value);
    }
    getDataContextItem(key) {
        let iv = this.i.bv(key);
        return (iv);
    }
    setExternalDataContextItem(key, value) {
        this.i.el(key, value);
    }
    getExternalDataContextItem(key) {
        let iv = this.i.bz(key);
        return (iv);
    }
    /**
     * Exports visual information about the current state of the grid.
    
    */
    exportVisualData() {
        let iv = this.i.bt();
        return (iv);
    }
    /**
     * Returns a serialized copy of the exported visual model
    
    */
    exportSerializedVisualData() {
        let iv = this.i.cg();
        return (iv);
    }
    /**
     * Event raised when a command is triggered in the toolbar.
    */
    get onCommand() {
        return this._onCommand;
    }
    set onCommand(ev) {
        if (this._onCommand_wrapped !== null) {
            this.i.onCommand = delegateRemove(this.i.onCommand, this._onCommand_wrapped);
            this._onCommand_wrapped = null;
            this._onCommand = null;
        }
        this._onCommand = ev;
        this._onCommand_wrapped = (o, e) => {
            let outerArgs = new IgrToolCommandEventArgs();
            outerArgs._provideImplementation(e);
            if (this.beforeOnCommand) {
                this.beforeOnCommand(this, outerArgs);
            }
            if (this._onCommand) {
                this._onCommand(this, outerArgs);
            }
        };
        this.i.onCommand = delegateCombine(this.i.onCommand, this._onCommand_wrapped);
        ;
    }
    /**
     * Event raised when a submenu is opened on the toolbar.
    */
    get subMenuOpening() {
        return this._subMenuOpening;
    }
    set subMenuOpening(ev) {
        if (this._subMenuOpening_wrapped !== null) {
            this.i.subMenuOpening = delegateRemove(this.i.subMenuOpening, this._subMenuOpening_wrapped);
            this._subMenuOpening_wrapped = null;
            this._subMenuOpening = null;
        }
        this._subMenuOpening = ev;
        this._subMenuOpening_wrapped = (o, e) => {
            let outerArgs = new IgrToolbarSubMenuOpeningEventArgs();
            outerArgs._provideImplementation(e);
            if (this.beforeSubMenuOpening) {
                this.beforeSubMenuOpening(this, outerArgs);
            }
            if (this._subMenuOpening) {
                this._subMenuOpening(this, outerArgs);
            }
        };
        this.i.subMenuOpening = delegateCombine(this.i.subMenuOpening, this._subMenuOpening_wrapped);
        ;
    }
    /**
     * Event raised when a submenu is closed on the toolbar
    */
    get subMenuClosing() {
        return this._subMenuClosing;
    }
    set subMenuClosing(ev) {
        if (this._subMenuClosing_wrapped !== null) {
            this.i.subMenuClosing = delegateRemove(this.i.subMenuClosing, this._subMenuClosing_wrapped);
            this._subMenuClosing_wrapped = null;
            this._subMenuClosing = null;
        }
        this._subMenuClosing = ev;
        this._subMenuClosing_wrapped = (o, e) => {
            let outerArgs = new IgrToolbarSubMenuClosingEventArgs();
            outerArgs._provideImplementation(e);
            if (this.beforeSubMenuClosing) {
                this.beforeSubMenuClosing(this, outerArgs);
            }
            if (this._subMenuClosing) {
                this._subMenuClosing(this, outerArgs);
            }
        };
        this.i.subMenuClosing = delegateCombine(this.i.subMenuClosing, this._subMenuClosing_wrapped);
        ;
    }
}
