import { __extends, __values } from "tslib";
import * as React from 'react';
import { stringToColor, colorToString, fromSize, ensureEnum, brushToString, stringToBrush, ensureBool, initializePropertiesFromCss, NamePatcher, isValidProp, toSpinal, getModifiedProps, fromBrushCollection, toBrushCollection } from "igniteui-react-core";
import { FontInfo } from "igniteui-react-core";
import { ReactRenderer, PortalManager } from "igniteui-react-core";
import { delegateCombine, delegateRemove } from "igniteui-react-core";
import { ContentChildrenManager } from "igniteui-react-core";
import { CollectionAdapter } from "igniteui-react-core";
import { NotifyCollectionChangedAction } from "igniteui-react-core";
import { ToolPanel } from './ToolPanel';
import { BaseControlTheme_$type } from "igniteui-react-core";
import { ControlDisplayDensity_$type } from "igniteui-react-core";
import { ToolPanelOrientation_$type } from './ToolPanelOrientation';
import { ToolAction } from "./ToolAction";
import { IgrToolActionCollection } from "./igr-tool-action-collection";
import { IgrToolPanelContentRefreshedEventArgs } from "./igr-tool-panel-content-refreshed-event-args";
import { IgrToolCommandEventArgs } from './igr-tool-command-event-args';
import { NestedActionMode_$type } from './NestedActionMode';
var IgrToolPanel = /** @class */ /*@__PURE__*/ (function (_super) {
    __extends(IgrToolPanel, _super);
    function IgrToolPanel(props) {
        var _this = _super.call(this, props) || this;
        _this._implementation = null;
        _this._reactRenderer = null;
        _this.contentActions = [];
        /**
        * The style mappings actually present in the treemap. Do not directly modify this array.
        * This array's contents can be modified by causing React to reproject the child content.
        * Or adding and removing ranges from the manual ranges collection on the ranges property.
        */
        _this.actualActions = [];
        _this._actions = null;
        _this._actionsAdapter = null;
        _this.__p = null;
        _this._hasUserValues = new Set();
        _this._stylingContainer = null;
        _this._stylingParent = null;
        _this._inStyling = false;
        _this._contentRefreshed = null;
        _this._contentRefreshed_wrapped = null;
        _this._onCommand = null;
        _this._onCommand_wrapped = null;
        if (_this._styling) {
            NamePatcher.ensureStylablePatched(Object.getPrototypeOf(_this));
        }
        _this._implementation = _this.createImplementation();
        _this._implementation.externalObject = _this;
        _this._getMainRef = _this._getMainRef.bind(_this);
        _this._contentChildrenManager = new ContentChildrenManager(function (ch) { return ch.key || ch.props.name; }, function (ch) { return ch.key || ch.props.name; }, function () { return _this._updateContentChildren(); });
        _this._actionsAdapter = new CollectionAdapter(_this.contentActions, _this.i.actions, _this.actualActions, function (c) { return c.i; }, function (i) {
            if (_this._reactRenderer && _this._elRef) {
                i._styling(_this._elRef, _this, _this);
            }
        }, function (i) { });
        //var multiColumnComboBox = this.i;
        //multiColumnComboBox.notifySizeChanged();
        if (props) {
            _this.initializeProperties();
        }
        _this._initialized = true;
        return _this;
    }
    Object.defineProperty(IgrToolPanel.prototype, "height", {
        get: function () {
            return this._height;
        },
        set: function (value) {
            this._height = value;
            if (this._elRef) {
                this._elRef.style.height = value;
                this.i.notifySizeChanged();
            }
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolPanel.prototype, "width", {
        get: function () {
            return this._width;
        },
        set: function (value) {
            this._width = value;
            if (this._elRef) {
                this._elRef.style.height = value;
                this.i.notifySizeChanged();
            }
        },
        enumerable: false,
        configurable: true
    });
    IgrToolPanel.prototype.ensureStretch = function () {
        this.width = "100%";
        this.height = "100%";
    };
    IgrToolPanel.prototype._getMainRef = function (ref) {
        this._elRef = ref;
        this.verifyReferences();
    };
    IgrToolPanel.prototype._updateContentChildren = function () {
        this.contentActions.length = 0;
        var contentChildrenActual = this._contentChildrenManager.contentChildrenActual;
        for (var i = 0; i < contentChildrenActual.length; i++) {
            if ((ToolAction.$type).isAssignableFrom(contentChildrenActual[i].i.$type)) {
                this.contentActions.push(contentChildrenActual[i]);
            }
        }
        if (this._actionsAdapter !== null) {
            this._actionsAdapter.notifyContentChanged();
        }
    };
    Object.defineProperty(IgrToolPanel.prototype, "actions", {
        /**
         * A collection of manually added style mappings for the treemap.
        */
        get: function () {
            var _this = this;
            if (this._actions === null) {
                var coll = new IgrToolActionCollection();
                var inner = coll._innerColl;
                inner.addListener(function (sender, e) {
                    switch (e.action) {
                        case NotifyCollectionChangedAction.Add:
                            _this._actionsAdapter.insertManualItem(e.newStartingIndex, e.newItems.item(0));
                            break;
                        case NotifyCollectionChangedAction.Remove:
                            _this._actionsAdapter.removeManualItemAt(e.oldStartingIndex);
                            break;
                        case NotifyCollectionChangedAction.Replace:
                            _this._actionsAdapter.removeManualItemAt(e.oldStartingIndex);
                            _this._actionsAdapter.insertManualItem(e.newStartingIndex, e.newItems.item(0));
                            break;
                        case NotifyCollectionChangedAction.Reset:
                            _this._actionsAdapter.clearManualItems();
                            break;
                    }
                });
                this._actions = coll;
            }
            return this._actions;
        },
        enumerable: false,
        configurable: true
    });
    IgrToolPanel.prototype.render = function () {
        var children = this._contentChildrenManager.getChildren(this.props.children);
        if (this._portalManager)
            this._portalManager.onRender(children);
        var div = React.createElement("div", {
            ref: this._getMainRef,
            className: "ig-tool-panel igr-tool-panel",
            children: children
        });
        return div;
    };
    IgrToolPanel.prototype.verifyReferences = function () {
        var _this = this;
        if (this._elRef) {
            this.requestRender = this.requestRender.bind(this);
            this._portalManager = new PortalManager("toolPanelContent", this.requestRender);
            this._portalManager.disableContentPortal = true;
            this._reactRenderer = new ReactRenderer(this._elRef, document, true, {}, this._portalManager);
            if (document) {
                this._elRef.style.display = "block";
                this._elRef.style.width = "100%";
                this._elRef.style.height = "100%";
            }
            // render needs to fire again after the portalManager is created. 
            this.requestRender();
            this.i.provideContainer(this._reactRenderer);
            this.i.notifySizeChanged();
            this._reactRenderer.addSizeWatcher(function () {
                _this.i.notifySizeChanged();
            });
        }
    };
    IgrToolPanel.prototype.requestRender = function () {
        if (this._initialized)
            this.setState({});
    };
    IgrToolPanel.prototype.shouldComponentUpdate = function (nextProps, nextState) {
        var e_1, _a;
        var mod = getModifiedProps(this.props, nextProps);
        try {
            for (var _b = __values(Object.keys(mod)), _c = _b.next(); !_c.done; _c = _b.next()) {
                var p = _c.value;
                if (isValidProp(this, p)) {
                    this[p] = mod[p];
                }
            }
        }
        catch (e_1_1) {
            e_1 = { error: e_1_1 };
        }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return))
                    _a.call(_b);
            }
            finally {
                if (e_1)
                    throw e_1.error;
            }
        }
        return true;
    };
    IgrToolPanel.prototype.initializeProperties = function () {
        var e_2, _a;
        try {
            for (var _b = __values(Object.keys(this.props)), _c = _b.next(); !_c.done; _c = _b.next()) {
                var p = _c.value;
                if (isValidProp(this, p)) {
                    this[p] = this.props[p];
                }
            }
        }
        catch (e_2_1) {
            e_2 = { error: e_2_1 };
        }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return))
                    _a.call(_b);
            }
            finally {
                if (e_2)
                    throw e_2.error;
            }
        }
    };
    // supports angular themes or custom properties set in CSS
    IgrToolPanel.prototype.updateStyle = function () {
        this._styling(this._elRef, this);
    };
    IgrToolPanel.prototype.componentWillUnmount = function () {
        this.i.destroy();
        if (this._reactRenderer)
            this._reactRenderer.destroy();
    };
    IgrToolPanel.prototype.componentDidMount = function () {
        this._elRef.style.width = this._width ? this._width : "";
        this._elRef.style.height = this._height ? this._height : "";
        this.i.notifySizeChanged();
        this.initializeContent();
    };
    IgrToolPanel.prototype.initializeContent = function () {
        this._styling(this._elRef, this);
        if (this.actualActions && this.actualActions.length > 0) {
            var currProperty = this.actualActions;
            for (var i = 0; i < currProperty.length; i++) {
                currProperty[i]._styling(this._elRef, this, this);
            }
        }
        this.updateStyle();
        this.i.notifySizeChanged();
    };
    IgrToolPanel.prototype.createImplementation = function () {
        return new ToolPanel();
    };
    Object.defineProperty(IgrToolPanel.prototype, "i", {
        get: function () {
            return this._implementation;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolPanel.prototype, "showOnHover", {
        /**
     * Gets or sets if the tool actions should display their submenus on mouse hover.
    */
        get: function () {
            return this.i.ay;
        },
        set: function (v) {
            this.i.ay = ensureBool(v);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolPanel.prototype, "showTooltipOnHover", {
        /**
         * Gets or sets if the tool action icon buttons should display their tooltips on mouse hover.
        */
        get: function () {
            return this.i.az;
        },
        set: function (v) {
            this.i.az = ensureBool(v);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolPanel.prototype, "dropdownClickBuffer", {
        /**
         * Gets or sets the amount of time to wait when clicking on the icon menu button while the dropdown is opening. If a click was
         * executed within this buffer window the dropdown will be left open. Otherwise it will close it. Setting a negative value will
         * always keep the dropdown open during a hover open and will require an extra click to close.
        */
        get: function () {
            return this.i.cp;
        },
        set: function (v) {
            this.i.cp = +v;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolPanel.prototype, "cellTextStyle", {
        /**
         * Gets or sets the text style for the panel.
        */
        get: function () {
            if (this.i.ak == null) {
                return null;
            }
            return this.i.ak.fontString;
        },
        set: function (v) {
            var fi = new FontInfo();
            fi.fontString = v;
            this.i.ak = fi;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolPanel.prototype, "subtitleTextStyle", {
        /**
         * Gets or sets the subtitle text style for the panel.
        */
        get: function () {
            if (this.i.am == null) {
                return null;
            }
            return this.i.am.fontString;
        },
        set: function (v) {
            var fi = new FontInfo();
            fi.fontString = v;
            this.i.am = fi;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolPanel.prototype, "baseTheme", {
        /**
         * Gets or sets the base built in theme to use for the checkbox.
        */
        get: function () {
            return this.i.x;
        },
        set: function (v) {
            this.i.x = ensureEnum(BaseControlTheme_$type, v);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolPanel.prototype, "density", {
        /**
         * Gets or sets the display density to use for the toolbar.
        */
        get: function () {
            return this.i.ac;
        },
        set: function (v) {
            this.i.ac = ensureEnum(ControlDisplayDensity_$type, v);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolPanel.prototype, "actualDensity", {
        /**
         * Gets the resolved display density to use for the toolbar.
        */
        get: function () {
            return this.i.ab;
        },
        set: function (v) {
            this.i.ab = ensureEnum(ControlDisplayDensity_$type, v);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolPanel.prototype, "backgroundColor", {
        /**
         * Gets or sets the color to use for the background of the component.
        */
        get: function () {
            return brushToString(this.i.iu);
        },
        set: function (v) {
            this.i.iu = stringToBrush(v);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolPanel.prototype, "actualBackgroundColor", {
        get: function () {
            return brushToString(this.i.h7);
        },
        set: function (v) {
            this.i.h7 = stringToBrush(v);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolPanel.prototype, "submenuBackgroundColor", {
        /**
         * Gets or sets the background color for submenus.
        */
        get: function () {
            return brushToString(this.i.ja);
        },
        set: function (v) {
            this.i.ja = stringToBrush(v);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolPanel.prototype, "actualSubmenuBackgroundColor", {
        get: function () {
            return brushToString(this.i.im);
        },
        set: function (v) {
            this.i.im = stringToBrush(v);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolPanel.prototype, "groupHeaderBackgroundColor", {
        /**
         * Gets or sets the group header background color.
        */
        get: function () {
            return brushToString(this.i.i0);
        },
        set: function (v) {
            this.i.i0 = stringToBrush(v);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolPanel.prototype, "actualGroupHeaderBackgroundColor", {
        get: function () {
            return brushToString(this.i.ib);
        },
        set: function (v) {
            this.i.ib = stringToBrush(v);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolPanel.prototype, "separatorBackgroundColor", {
        /**
         * Gets or sets the separator background color.
        */
        get: function () {
            return brushToString(this.i.i9);
        },
        set: function (v) {
            this.i.i9 = stringToBrush(v);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolPanel.prototype, "actualSeparatorBackgroundColor", {
        get: function () {
            return brushToString(this.i.il);
        },
        set: function (v) {
            this.i.il = stringToBrush(v);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolPanel.prototype, "groupHeaderSeparatorBackgroundColor", {
        /**
         * Gets or sets the group header separator background color.
        */
        get: function () {
            return brushToString(this.i.i1);
        },
        set: function (v) {
            this.i.i1 = stringToBrush(v);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolPanel.prototype, "actualGroupHeaderSeparatorBackgroundColor", {
        get: function () {
            return brushToString(this.i.ic);
        },
        set: function (v) {
            this.i.ic = stringToBrush(v);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolPanel.prototype, "hoverBackgroundColor", {
        /**
         * Gets or sets the hover background color.
        */
        get: function () {
            return brushToString(this.i.i5);
        },
        set: function (v) {
            this.i.i5 = stringToBrush(v);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolPanel.prototype, "actualHoverBackgroundColor", {
        get: function () {
            return brushToString(this.i.ih);
        },
        set: function (v) {
            this.i.ih = stringToBrush(v);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolPanel.prototype, "textColor", {
        /**
         * Gets or sets the color to use for the text of the component.
        */
        get: function () {
            return brushToString(this.i.jc);
        },
        set: function (v) {
            this.i.jc = stringToBrush(v);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolPanel.prototype, "actualTextColor", {
        get: function () {
            return brushToString(this.i.ip);
        },
        set: function (v) {
            this.i.ip = stringToBrush(v);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolPanel.prototype, "actualSubtitleTextColor", {
        get: function () {
            return brushToString(this.i.io);
        },
        set: function (v) {
            this.i.io = stringToBrush(v);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolPanel.prototype, "subtitleTextColor", {
        /**
         * Gets or sets the color to use for the subtitle of the component.
        */
        get: function () {
            return brushToString(this.i.jb);
        },
        set: function (v) {
            this.i.jb = stringToBrush(v);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolPanel.prototype, "disabledTextColor", {
        /**
         * Gets or sets the color to use for the text in disabled tool actions.
        */
        get: function () {
            return brushToString(this.i.ix);
        },
        set: function (v) {
            this.i.ix = stringToBrush(v);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolPanel.prototype, "actualDisabledTextColor", {
        get: function () {
            return brushToString(this.i.ia);
        },
        set: function (v) {
            this.i.ia = stringToBrush(v);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolPanel.prototype, "groupHeaderTextColor", {
        /**
         * Gets or sets the text color for group header tool actions.
        */
        get: function () {
            return brushToString(this.i.i3);
        },
        set: function (v) {
            this.i.i3 = stringToBrush(v);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolPanel.prototype, "actualGroupHeaderTextColor", {
        get: function () {
            return brushToString(this.i.ie);
        },
        set: function (v) {
            this.i.ie = stringToBrush(v);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolPanel.prototype, "groupHeaderSubtitleTextColor", {
        /**
         * Gets or sets the text color for group header subtitle.
        */
        get: function () {
            return brushToString(this.i.i2);
        },
        set: function (v) {
            this.i.i2 = stringToBrush(v);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolPanel.prototype, "actualGroupHeaderSubtitleTextColor", {
        get: function () {
            return brushToString(this.i.id);
        },
        set: function (v) {
            this.i.id = stringToBrush(v);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolPanel.prototype, "toolTipTextColor", {
        /**
         * Gets or sets the tooltip text color.
        */
        get: function () {
            return brushToString(this.i.je);
        },
        set: function (v) {
            this.i.je = stringToBrush(v);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolPanel.prototype, "actualToolTipTextColor", {
        get: function () {
            return brushToString(this.i.ir);
        },
        set: function (v) {
            this.i.ir = stringToBrush(v);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolPanel.prototype, "toolTipBackgroundColor", {
        /**
         * Gets or sets the tooltip background color.
        */
        get: function () {
            return brushToString(this.i.jd);
        },
        set: function (v) {
            this.i.jd = stringToBrush(v);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolPanel.prototype, "actualToolTipBackgroundColor", {
        /**
         * Gets the actual tooltip background color.
        */
        get: function () {
            return brushToString(this.i.iq);
        },
        set: function (v) {
            this.i.iq = stringToBrush(v);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolPanel.prototype, "toolTipCornerRadius", {
        /**
         * Gets or sets the tooltip corner radius.
        */
        get: function () {
            return this.i.ci;
        },
        set: function (v) {
            this.i.ci = +v;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolPanel.prototype, "actualToolTipCornerRadius", {
        /**
         * Gets the actual tooltip corner radius.
        */
        get: function () {
            return this.i.b4;
        },
        set: function (v) {
            this.i.b4 = +v;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolPanel.prototype, "toolTipElevation", {
        /**
         * Gets or sets the shadow elevation for the tooltip.
        */
        get: function () {
            return this.i.cj;
        },
        set: function (v) {
            this.i.cj = +v;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolPanel.prototype, "actualToolTipElevation", {
        /**
         * Gets the actual shadow elevation used by the tooltip.
        */
        get: function () {
            return this.i.b5;
        },
        set: function (v) {
            this.i.b5 = +v;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolPanel.prototype, "iconFill", {
        /**
         * Gets or sets the fill color of icons in the ToolPanel.
        */
        get: function () {
            return brushToString(this.i.i6);
        },
        set: function (v) {
            this.i.i6 = stringToBrush(v);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolPanel.prototype, "actualIconFill", {
        /**
         * Gets the actual icon fill color.
        */
        get: function () {
            return brushToString(this.i.ii);
        },
        set: function (v) {
            this.i.ii = stringToBrush(v);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolPanel.prototype, "iconStroke", {
        /**
         * Gets or sets the stroke color of icons in the ToolPanel.
        */
        get: function () {
            return brushToString(this.i.i7);
        },
        set: function (v) {
            this.i.i7 = stringToBrush(v);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolPanel.prototype, "actualIconStroke", {
        /**
         * Gets the actual icon stroke color.
        */
        get: function () {
            return brushToString(this.i.ij);
        },
        set: function (v) {
            this.i.ij = stringToBrush(v);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolPanel.prototype, "separatorVerticalPaddingLeft", {
        /**
         * Gets or sets the left padding for separators in the vertical orientation.
        */
        get: function () {
            return this.i.cf;
        },
        set: function (v) {
            this.i.cf = +v;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolPanel.prototype, "actualSeparatorVerticalPaddingLeft", {
        get: function () {
            return this.i.b1;
        },
        set: function (v) {
            this.i.b1 = +v;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolPanel.prototype, "separatorVerticalPaddingRight", {
        /**
         * Gets or sets the right padding for separators in the vertical orientation.
        */
        get: function () {
            return this.i.cg;
        },
        set: function (v) {
            this.i.cg = +v;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolPanel.prototype, "actualSeparatorVerticalPaddingRight", {
        get: function () {
            return this.i.b2;
        },
        set: function (v) {
            this.i.b2 = +v;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolPanel.prototype, "separatorVerticalPaddingTop", {
        /**
         * Gets or sets the top padding for separators in the vertical orientation.
        */
        get: function () {
            return this.i.ch;
        },
        set: function (v) {
            this.i.ch = +v;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolPanel.prototype, "actualSeparatorVerticalPaddingTop", {
        get: function () {
            return this.i.b3;
        },
        set: function (v) {
            this.i.b3 = +v;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolPanel.prototype, "separatorVerticalPaddingBottom", {
        /**
         * Gets or sets the bottom padding for separators in the vertical orientation.
        */
        get: function () {
            return this.i.ce;
        },
        set: function (v) {
            this.i.ce = +v;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolPanel.prototype, "actualSeparatorVerticalPaddingBottom", {
        get: function () {
            return this.i.b0;
        },
        set: function (v) {
            this.i.b0 = +v;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolPanel.prototype, "separatorHorizontalPaddingLeft", {
        /**
         * Gets or sets the left padding for separators in the horizontal orientation.
        */
        get: function () {
            return this.i.cb;
        },
        set: function (v) {
            this.i.cb = +v;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolPanel.prototype, "actualSeparatorHorizontalPaddingLeft", {
        get: function () {
            return this.i.bx;
        },
        set: function (v) {
            this.i.bx = +v;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolPanel.prototype, "separatorHorizontalPaddingRight", {
        /**
         * Gets or sets the right padding for separators in the horizontal orientation.
        */
        get: function () {
            return this.i.cc;
        },
        set: function (v) {
            this.i.cc = +v;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolPanel.prototype, "actualSeparatorHorizontalPaddingRight", {
        get: function () {
            return this.i.by;
        },
        set: function (v) {
            this.i.by = +v;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolPanel.prototype, "separatorHorizontalPaddingTop", {
        /**
         * Gets or sets the top padding for separators in the horizontal orientation.
        */
        get: function () {
            return this.i.cd;
        },
        set: function (v) {
            this.i.cd = +v;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolPanel.prototype, "actualSeparatorHorizontalPaddingTop", {
        get: function () {
            return this.i.bz;
        },
        set: function (v) {
            this.i.bz = +v;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolPanel.prototype, "separatorHorizontalPaddingBottom", {
        /**
         * Gets or sets the bottom padding for separators in the horizontal orientation.
        */
        get: function () {
            return this.i.ca;
        },
        set: function (v) {
            this.i.ca = +v;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolPanel.prototype, "actualSeparatorHorizontalPaddingBottom", {
        get: function () {
            return this.i.bw;
        },
        set: function (v) {
            this.i.bw = +v;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolPanel.prototype, "orientation", {
        /**
         * Gets or sets the color to use for the background of the component.
        */
        get: function () {
            return this.i.u;
        },
        set: function (v) {
            this.i.u = ensureEnum(ToolPanelOrientation_$type, v);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolPanel.prototype, "rowHeight", {
        /**
         * Gets or sets the row height for the panel.
        */
        get: function () {
            return this.i.ct;
        },
        set: function (v) {
            this.i.ct = +v;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolPanel.prototype, "actualRowHeight", {
        /**
         * Gets the actual row height used by the panel.
        */
        get: function () {
            return this.i.bv;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolPanel.prototype, "highlightWidth", {
        get: function () {
            return this.i.b9;
        },
        set: function (v) {
            this.i.b9 = +v;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolPanel.prototype, "actualHighlightWidth", {
        get: function () {
            return this.i.bu;
        },
        set: function (v) {
            this.i.bu = +v;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolPanel.prototype, "highlightRadius", {
        get: function () {
            return this.i.b8;
        },
        set: function (v) {
            this.i.b8 = +v;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolPanel.prototype, "actualHighlightRadius", {
        get: function () {
            return this.i.bt;
        },
        set: function (v) {
            this.i.bt = +v;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolPanel.prototype, "highlightColor", {
        get: function () {
            return brushToString(this.i.i4);
        },
        set: function (v) {
            this.i.i4 = stringToBrush(v);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolPanel.prototype, "actualHighlightColor", {
        get: function () {
            return brushToString(this.i.ig);
        },
        set: function (v) {
            this.i.ig = stringToBrush(v);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolPanel.prototype, "menuArrowStroke", {
        /**
         * Gets or sets the stroke color for all icon menu dropdown arrows.
        */
        get: function () {
            return brushToString(this.i.i8);
        },
        set: function (v) {
            this.i.i8 = stringToBrush(v);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolPanel.prototype, "actualMenuArrowStroke", {
        /**
         * Gets the actual stroke color for all icon menu dropdown arrows.
        */
        get: function () {
            return brushToString(this.i.ik);
        },
        set: function (v) {
            this.i.ik = stringToBrush(v);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolPanel.prototype, "uncheckedBackgroundColor", {
        /**
         * Gets or sets the background color of the radio when in the unchecked state.
        */
        get: function () {
            return brushToString(this.i.jf);
        },
        set: function (v) {
            this.i.jf = stringToBrush(v);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolPanel.prototype, "actualUncheckedBackgroundColor", {
        /**
         * Gets the actual background color of the radio when in the unchecked state.
        */
        get: function () {
            return brushToString(this.i.is);
        },
        set: function (v) {
            this.i.is = stringToBrush(v);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolPanel.prototype, "checkedBackgroundColor", {
        /**
         * Gets or sets the background color of the radio when in the checked state.
        */
        get: function () {
            return brushToString(this.i.iv);
        },
        set: function (v) {
            this.i.iv = stringToBrush(v);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolPanel.prototype, "actualCheckedBackgroundColor", {
        /**
         * Gets the actual background color of the radio when in the checked state.
        */
        get: function () {
            return brushToString(this.i.h8);
        },
        set: function (v) {
            this.i.h8 = stringToBrush(v);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolPanel.prototype, "uncheckedBorderColor", {
        /**
         * Gets or sets the border color of the radio when in the unchecked state.
        */
        get: function () {
            return brushToString(this.i.jg);
        },
        set: function (v) {
            this.i.jg = stringToBrush(v);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolPanel.prototype, "actualUncheckedBorderColor", {
        /**
         * Gets the actual border color of the radio when in the unchecked state.
        */
        get: function () {
            return brushToString(this.i.it);
        },
        set: function (v) {
            this.i.it = stringToBrush(v);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolPanel.prototype, "checkedBorderColor", {
        /**
         * Gets or sets the border color of the radio when in the checked state.
        */
        get: function () {
            return brushToString(this.i.iw);
        },
        set: function (v) {
            this.i.iw = stringToBrush(v);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolPanel.prototype, "actualCheckedBorderColor", {
        /**
         * Gets the actual border color of the radio when in the checked state.
        */
        get: function () {
            return brushToString(this.i.h9);
        },
        set: function (v) {
            this.i.h9 = stringToBrush(v);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolPanel.prototype, "dropdownDelay", {
        /**
         * Gets or sets the dropdown delay time in milliseconds.
        */
        get: function () {
            return this.i.cq;
        },
        set: function (v) {
            this.i.cq = +v;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolPanel.prototype, "actualDropdownDelay", {
        /**
         * Gets the actual dropdown delay.
        */
        get: function () {
            return this.i.co;
        },
        set: function (v) {
            this.i.co = +v;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolPanel.prototype, "nestedActionMode", {
        /**
         * Gets or sets the mode used when displaying nested actions.
        */
        get: function () {
            return this.i.g;
        },
        set: function (v) {
            this.i.g = ensureEnum(NestedActionMode_$type, v);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolPanel.prototype, "isOpen", {
        get: function () {
            return this.i.ax;
        },
        enumerable: false,
        configurable: true
    });
    IgrToolPanel.prototype.findByName = function (name) {
        if (this.findEphemera) {
            if (name && name.indexOf("@@e:") == 0) {
                return this.findEphemera(name);
            }
        }
        if (this.actions != null && this.actions.findByName && this.actions.findByName(name)) {
            return this.actions.findByName(name);
        }
        return null;
    };
    Object.defineProperty(IgrToolPanel.prototype, "hasUserValues", {
        get: function () {
            return this._hasUserValues;
        },
        enumerable: false,
        configurable: true
    });
    IgrToolPanel.prototype.__m = function (propertyName) {
        if (!this._inStyling) {
            this._hasUserValues.add(propertyName);
        }
    };
    IgrToolPanel.prototype._styling = function (container, component, parent) {
        if (this._inStyling) {
            return;
        }
        this._inStyling = true;
        this._stylingContainer = container;
        this._stylingParent = component;
        var genericPrefix = "";
        var typeName = this.i.$type.name;
        if (typeName.indexOf("Xam") === 0) {
            typeName = typeName.substring(3);
        }
        genericPrefix = toSpinal("ToolPanel");
        var additionalPrefixes = [];
        var prefix = toSpinal(typeName);
        additionalPrefixes.push(prefix + "-");
        var b = this.i.$type.baseType;
        while (b && b.name != "Object" &&
            b.name != "Base" &&
            b.name != "Control" &&
            b.Name != "DependencyObject" &&
            b.Name != "FrameworkElement") {
            typeName = b.name;
            if (typeName.indexOf("Xam") === 0) {
                typeName = typeName.substring(3);
            }
            var basePrefix = toSpinal(typeName);
            additionalPrefixes.push(basePrefix + "-");
            b = b.baseType;
        }
        if (parent) {
            var parentTypeName = parent.i.$type.name;
            if (parentTypeName.indexOf("Xam") === 0) {
                parentTypeName = parentTypeName.substring(3);
            }
            var parentPrefix = toSpinal(parentTypeName);
            additionalPrefixes.push(parentPrefix + "-" + genericPrefix + "-");
            additionalPrefixes.push(parentPrefix + "-" + prefix + "-");
        }
        initializePropertiesFromCss(container, this, genericPrefix + "-", this.hasUserValues, false, additionalPrefixes);
        if (this._otherStyling) {
            this._otherStyling(container, component, parent);
        }
        this._inStyling = false;
    };
    IgrToolPanel.prototype.refresh = function () {
        this.i.f9();
    };
    IgrToolPanel.prototype.getDesiredSize = function () {
        var iv = this.i.jj();
        return fromSize(iv);
    };
    IgrToolPanel.prototype.flushRefresh = function () {
        this.i.fr();
    };
    /**
     * Forces any open sub menus to close.
    
    */
    IgrToolPanel.prototype.closeSubmenus = function () {
        this.i.fm();
    };
    IgrToolPanel.prototype.exportVisualData = function () {
        var iv = this.i.cu();
        return (iv);
    };
    IgrToolPanel.prototype.exportSerializedVisualData = function () {
        var iv = this.i.ee();
        return (iv);
    };
    IgrToolPanel.prototype.setBrushContextItem = function (key, value) {
        this.i.gi(key, stringToBrush(value));
    };
    IgrToolPanel.prototype.getBrushContextItem = function (key) {
        var iv = this.i.iy(key);
        return brushToString(iv);
    };
    IgrToolPanel.prototype.setBrushCollectionContextItem = function (key, value) {
        this.i.gg(key, toBrushCollection(value));
    };
    IgrToolPanel.prototype.getBrushCollectionContextItem = function (key) {
        var iv = this.i.a(key);
        return fromBrushCollection(iv);
    };
    IgrToolPanel.prototype.setDoubleContextItem = function (key, value) {
        this.i.gq(key, value);
    };
    IgrToolPanel.prototype.getDoubleContextItem = function (key) {
        var iv = this.i.b6(key);
        return (iv);
    };
    IgrToolPanel.prototype.setIntContextItem = function (key, value) {
        this.i.gs(key, value);
    };
    IgrToolPanel.prototype.getIntContextItem = function (key) {
        var iv = this.i.cr(key);
        return (iv);
    };
    IgrToolPanel.prototype.setStringContextItem = function (key, value) {
        this.i.gu(key, value);
    };
    IgrToolPanel.prototype.getStringContextItem = function (key) {
        var iv = this.i.ef(key);
        return (iv);
    };
    IgrToolPanel.prototype.setBoolContextItem = function (key, value) {
        this.i.ge(key, value);
    };
    IgrToolPanel.prototype.getBoolContextItem = function (key) {
        var iv = this.i.au(key);
        return (iv);
    };
    IgrToolPanel.prototype.setColorContextItem = function (key, value) {
        this.i.gk(key, stringToColor(value));
    };
    IgrToolPanel.prototype.getColorContextItem = function (key) {
        var iv = this.i.jh(key);
        return colorToString(iv);
    };
    IgrToolPanel.prototype.setDataContextItem = function (key, value) {
        this.i.go(key, value);
    };
    IgrToolPanel.prototype.getDataContextItem = function (key) {
        var iv = this.i.cx(key);
        return (iv);
    };
    Object.defineProperty(IgrToolPanel.prototype, "contentRefreshed", {
        get: function () {
            return this._contentRefreshed;
        },
        set: function (ev) {
            var _this = this;
            if (this._contentRefreshed_wrapped !== null) {
                this.i.contentRefreshed = delegateRemove(this.i.contentRefreshed, this._contentRefreshed_wrapped);
                this._contentRefreshed_wrapped = null;
                this._contentRefreshed = null;
            }
            this._contentRefreshed = ev;
            this._contentRefreshed_wrapped = function (o, e) {
                var outerArgs = new IgrToolPanelContentRefreshedEventArgs();
                outerArgs._provideImplementation(e);
                if (_this.beforeContentRefreshed) {
                    _this.beforeContentRefreshed(_this, outerArgs);
                }
                if (_this._contentRefreshed) {
                    _this._contentRefreshed(_this, outerArgs);
                }
            };
            this.i.contentRefreshed = delegateCombine(this.i.contentRefreshed, this._contentRefreshed_wrapped);
            ;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgrToolPanel.prototype, "onCommand", {
        get: function () {
            return this._onCommand;
        },
        set: function (ev) {
            var _this = this;
            if (this._onCommand_wrapped !== null) {
                this.i.onCommand = delegateRemove(this.i.onCommand, this._onCommand_wrapped);
                this._onCommand_wrapped = null;
                this._onCommand = null;
            }
            this._onCommand = ev;
            this._onCommand_wrapped = function (o, e) {
                var outerArgs = new IgrToolCommandEventArgs();
                outerArgs._provideImplementation(e);
                if (_this.beforeOnCommand) {
                    _this.beforeOnCommand(_this, outerArgs);
                }
                if (_this._onCommand) {
                    _this._onCommand(_this, outerArgs);
                }
            };
            this.i.onCommand = delegateCombine(this.i.onCommand, this._onCommand_wrapped);
            ;
        },
        enumerable: false,
        configurable: true
    });
    return IgrToolPanel;
}(React.Component));
export { IgrToolPanel };
